@@ -1,5 +1,7 @@
+#!/usr/bin/env python
 import os
 import sys, string
+from os.path import isfile
 
 table = [
 	["away.png", "away"],
@@ -13,10 +15,13 @@ table = [
 	["hilite2.png", "hilite2"],
 	["nicotine+.png", "nicotinen"],
 	["n.png", "n"],
+	["notify.png", "notify"],
 ]
 flagtable = []
 for name in os.listdir(os.path.join("img", "geoip")):
-	flagtable.append( (os.path.join("img", "geoip", name), 'flag_%s' % name[:2].upper()) )
+	p = os.path.join("img", "geoip", name)
+	if isfile(p):
+		flagtable.append((os.path.join("img", "geoip", name), 'flag_%s' % name[:2].upper()))
 
 outf = open(os.path.join("pynicotine","gtkgui","imagedata.py"), "w")
 for image in table:
