@@ -201,7 +201,7 @@ class AboutCreditsDialog(GenericAboutDialog):
 		text = _("""daelstorm
 \t(Lead developer)
 \t<daelstorm@gmail.com>
-\thttp://thegraveyard.org/daelstorm/
+\thttp://daelstorm.thegraveyard.org/
 gallows
 \t(Developer, packager)
 \t<g4ll0ws@gmail.com>
@@ -290,7 +290,7 @@ Finnish
 Euskara
  * The Librezale.org Team <librezale@librezale.org> http://librezale.org
  """)
- 		AppendLine(self.TransTextView, text, None, None, showstamp=False)
+		AppendLine(self.TransTextView, text, None, None, showstamp=False)
 		
 		translatorsLabel = gtk.Label(_("Translators"))
 		translatorsLabel.show()
@@ -436,7 +436,7 @@ class AboutRoomsDialog(GenericTableDialog):
 		"", "",
 		"<b>"+_("Search")+"</b>", "",
 		"/search /s '" + _("query")+"'", _("Start a new search for 'query'"),
-		"/rsearch /rs '" + _("query")+"'", _("Search the joined roms for 'query'"),
+		"/rsearch /rs '" + _("query")+"'", _("Search the joined rooms for 'query'"),
 		"/bsearch /bs '" + _("query")+"'", _("Search the buddy list for 'query'"),
 		"/usearch /us '" + _("user")+"' '" + _("query")+"'", _("Search a user's shares for 'query'"),
 		"", "",
@@ -510,4 +510,4 @@ class AboutDependenciesDialog(GenericTableDialog):
 
 	def __init__(self, parent):
 		GenericTableDialog.__init__(self, parent, _("About optional dependencies"))
-		self.table.set_row_spacings(5)
\ No newline at end of file
+		self.table.set_row_spacings(5)
