@@ -27,8 +27,8 @@ from utils import recode, InputDialog
 from pynicotine.utils import _
 
 
-def ChooseDir(parent = None, initialdir = "~", create=False, name=None):
-	dialog = gtk.FileChooserDialog(parent=parent, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
+def ChooseDir(parent = None, initialdir = "~", create=False, name=None, title=None):
+	dialog = gtk.FileChooserDialog(title=title, parent=parent, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 	if create:
 		dialog.set_action(gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER)
 	else:
@@ -54,7 +54,7 @@ def ChooseDir(parent = None, initialdir = "~", create=False, name=None):
 	return res
 
 def ChooseFile(parent = None, initialdir = "~", initialfile = ""):
-	dialog = gtk.FileChooserDialog(parent=parent, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
+	dialog = gtk.FileChooserDialog(parent=parent, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 	dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
 	dialog.set_select_multiple(False)
 	dir = os.path.expanduser(initialdir)
@@ -73,7 +73,30 @@ def ChooseFile(parent = None, initialdir = "~", initialfile = ""):
 
 	return res
 
-def ChooseImage(parent = None, initialdir = "~", initialfile = ""):
+def SaveFile(parent = None, initialdir = "~", initialfile = "", title=None):
+	dialog = gtk.FileChooserDialog(parent=parent, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
+	dialog.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
+	dialog.set_select_multiple(False)
+	dialog.set_show_hidden(True)
+	if title:
+		dialog.set_title(title)
+	dir = os.path.expanduser(initialdir)
+	if os.path.exists(dir):
+		dialog.set_current_folder(dir)
+	else:
+		dialog.set_current_folder(os.path.expanduser("~"))
+	response = dialog.run()
+	
+	if response == gtk.RESPONSE_ACCEPT:
+		res = dialog.get_filenames()
+	else:
+		res = None
+
+	dialog.destroy()
+
+	return res
+
+def ChooseImage(parent = None, initialdir = "~", initialfile = "", title=None):
 	image = gtk.Image()
 	preview = gtk.ScrolledWindow()
 	preview.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -81,7 +104,7 @@ def ChooseImage(parent = None, initialdir = "~", initialfile = ""):
 	preview.add_with_viewport(image)
 	image.show()
 	preview.show()
-	dialog = gtk.FileChooserDialog(parent=parent, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
+	dialog = gtk.FileChooserDialog(title=title, parent=parent, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 	dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
 	dialog.set_select_multiple(False)
 	dialog.set_preview_widget(preview)
@@ -121,4 +144,4 @@ def update_preview_cb(file_chooser, preview):
 if __name__ == "__main__":
 	print ChooseDir()
 	print ChooseFile()
-	
\ No newline at end of file
+	
