@@ -123,7 +123,9 @@ class MetaDialog( gtk.Dialog):
 		self.Country.hide()
 		
 		hbox4.pack_start(self.Country, False, False)
-
+		self.Flag = gtk.Image()
+		self.Flag.hide()
+		hbox4.pack_start(self.Flag, False, False)
 		
 		self.buttonbox = gtk.HBox(False, 2)
 		self.buttonbox.show()
@@ -220,6 +222,7 @@ class MetaDialog( gtk.Dialog):
 			self.Immediate.hide()
 			self.Position.hide()
 			self.Country.hide()
+			self.Flag.hide()
 			self.Queue.hide()
 			self.Immediate.hide()
 			self.ImmediateLabel.hide()
@@ -232,6 +235,7 @@ class MetaDialog( gtk.Dialog):
 			self.Immediate.show()
 			self.Position.show()
 			self.Country.show()
+			self.Flag.show()
 			self.Queue.show()
 			self.Immediate.show()
 			self.ImmediateLabel.show()
@@ -242,7 +246,7 @@ class MetaDialog( gtk.Dialog):
 			self.DownloadAll.show()
 			
 		self.current = item
-		
+		data = self.data[self.current]
 		More = False
 		if len(self.data.keys()) > 1:
 			More = True
@@ -250,33 +254,37 @@ class MetaDialog( gtk.Dialog):
 		self.Previous.set_sensitive(More)
 		self.DownloadAll.set_sensitive(More)
 		
-		self.Username.set_text	(self.data[self.current]["user"])
-		self.Filename.set_text	(self.data[self.current]["filename"])
-		self.Directory.set_text	(self.data[self.current]["directory"])
-		self.Size.set_text	(str(self.data[self.current]["size"]))
-		self.Speed.set_text	(self.data[self.current]["speed"])
-		self.Position.set_text	(str(self.data[self.current]["position"]))
-		if self.data[self.current]["bitrate"] not in ("", None):
-			self.Bitrate.set_text(self.data[self.current]["bitrate"])
+		self.Username.set_text(data["user"])
+		self.Filename.set_text(data["filename"])
+		self.Directory.set_text(data["directory"])
+		self.Size.set_text(str(data["size"]))
+		self.Speed.set_text(data["speed"])
+		self.Position.set_text(str(data["position"]))
+		if data["bitrate"] not in ("", None):
+			self.Bitrate.set_text(data["bitrate"])
 		else:
 			self.Bitrate.set_text("")
-		self.Length.set_text	(self.data[self.current]["length"])
-		self.Queue.set_text	(self.data[self.current]["queue"])
-		self.Immediate.set_text	(str(self.data[self.current]["immediate"] == "Y"))
-		
-		if self.data[self.current]["country"] not in ("", None):
-			self.Country.set_markup(_("<b>Country Code:</b> ")+self.data[self.current]["country"] )
+		self.Length.set_text(data["length"])
+		self.Queue.set_text(data["queue"])
+		self.Immediate.set_text(str(data["immediate"] == "Y"))
+		country =  data["country"]
+		if country not in ("", None):
+			self.Country.set_markup(_("<b>Country Code:</b> ") + country )
 			self.Country.show()
+			self.Flag.set_from_pixbuf(self.nicotine.GetFlagImage("flag_" + country) )
+			self.Flag.show()
 		else:
 			self.Country.set_text("")
 			self.Country.hide()
+			self.Flag.set_from_pixbuf(None)
+			self.Flag.hide()
 		
 	def quit(self, w=None, event=None):
 		self.hide()
 		self.destroy()
 		gtk.main_quit()
+		
 	def click(self, button):
-
 		self.quit()
 	
 	def MakeLabel(self, parent, labeltitle, expand=True, fill=False, xalign=0):
@@ -287,7 +295,8 @@ class MetaDialog( gtk.Dialog):
 		#label.set_property("selectable", True)
 		parent.pack_start(label, expand, fill)
 
-		try:label.set_property("xalign", xalign)
+		try:
+			label.set_property("xalign", xalign)
 		except Exception, e:
 			print e
 			pass
@@ -358,10 +367,12 @@ class EntryDialog( gtk.Dialog):
 		self.action_area.pack_start(button)
 		button.show()
 		self.ret = None
+		
 	def quit(self, w=None, event=None):
 		self.hide()
 		self.destroy()
 		gtk.main_quit()
+		
 	def click(self, button):
 		if self.gotoption:
 			self.ret = [self.combo.child.get_text(), self.option.get_active()]
@@ -481,9 +492,9 @@ class FolderDownloadDialog( gtk.Dialog):
 		hbox2.show()
 		box.pack_start(hbox2)	
 
+		cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 		ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
 		ok_button.grab_default()
-		cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 		
 	def quit(self, *args):
 		self.destroy()
@@ -491,7 +502,7 @@ class FolderDownloadDialog( gtk.Dialog):
 def QuitBox(frame, title="Option Box", message="", default_text='', 
 	modal= True, status=None, tray=False, third="" ):
 	
-	win = OptionDialog(frame, message, modal=modal, status=status, option=tray, third=third)
+	win = OptionDialog(frame, message, modal=modal, status=status, option=tray, third=third, rememberbox=True)
 	win.connect("response", frame.on_quit_response)
 	win.set_title(title)
 	win.set_icon(frame.images["n"])
@@ -508,8 +519,8 @@ def ClearDialog(frame, title=_("Nicotine+"), message="", default_text='',
 	return win
 
 
-class OptionDialog( gtk.Dialog):
-	def __init__(self, frame, message="",modal= False, status=None, option=False, third=""):
+class OptionDialog(gtk.Dialog):
+	def __init__(self, frame, message="",modal= False, status=None, option=False, third="", rememberbox=False):
 		gtk.Dialog.__init__(self)
 		self.connect("destroy", self.quit)
 		self.connect("delete-event", self.quit)
@@ -542,10 +553,14 @@ class OptionDialog( gtk.Dialog):
 		hbox2 = gtk.HBox(spacing=5)
 		hbox2.set_border_width(5)
 		hbox2.show()
-		box.pack_start(hbox2)	
+		box.pack_start(hbox2)
+
+		# Storing under self. so we can find it easily later 
+		if rememberbox:
+			self.checkbox = gtk.CheckButton("Remember choice")
+			box.pack_start(self.checkbox)
+			self.checkbox.show()
 		
-		ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
-		ok_button.grab_default()
 		if option:
 			
 			Alignment = gtk.Alignment(0.5, 0.5, 0, 0)
@@ -573,6 +588,8 @@ class OptionDialog( gtk.Dialog):
 			tray_button.add(Alignment)
 
 		cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+		ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+		ok_button.grab_default()
 
 	def quit(self, *args):
 		self.destroy()
