@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2007 daelstorm. All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,28 +17,34 @@
 # Original copyright below
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
-import os
+import os, sys
 import gtk, gobject, pango
-import sets
-from nicotine_glade import PrivateChatTab
+
+from time import daylight, altzone
+
 from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, is_alias, EncodingsMenu, SaveEncoding, fixpath
 from chatrooms import GetCompletion
 from pynicotine import slskmessages
 from pynicotine.utils import _, version
 
 class PrivateChats(IconNotebook):
+	CMDS = set(["/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
+		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/ignoreip ", "/unban ", "/unignore ", "/clear ",
+		"/quit ", "/exit ", "/rescan ", "/nsa ", "/info ", "/ctcpversion ", "/join "])
+	
 	def __init__(self, frame):
 		ui = frame.np.config.sections["ui"]
-		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"])
+		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"], ui["tab_icons"], ui["tab_reorderable"])
 		self.popup_enable()
 
 		self.frame = frame
 		self.connected = 1
 		self.users = {}
+		self.clist = []
 		self.connect("switch-page", self.OnSwitchPage)
 
 	def OnSwitchPage(self, notebook, page, page_num, force = 0):
-		if self.frame.MainNotebook.get_current_page() != 1 and not force:
+		if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) and not force:
 			return
 		page = notebook.get_nth_page(page_num)
 		
@@ -49,7 +56,7 @@ class PrivateChats(IconNotebook):
 					self.frame.ClearNotification("private", tab.user)
 					
 	def ClearNotifications(self):
-		if self.frame.MainNotebook.get_current_page() != 1:
+		if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox):
 			return
 		page = self.get_nth_page( self.get_current_page())
 		for user, tab in self.users.items():
@@ -100,17 +107,23 @@ class PrivateChats(IconNotebook):
 			("$" + _("Add user to list"), popup.OnAddToList),
 			("$" + _("Ban this user"), popup.OnBanUser),
 			("$" + _("Ignore this user"), popup.OnIgnoreUser),
+			("$" + _("B_lock this user's IP Address"), popup.OnBlockUser),
+			("$" + _("Ignore this user's IP Address"), popup.OnIgnoreIP),
 			("", None),
 			("#" + _("Detach this tab"), self.users[user].Detach, gtk.STOCK_REDO),
 			("#" + _("Close this tab"), self.users[user].OnClose, gtk.STOCK_CLOSE),
 		)
 		popup.set_user(user)
-		
+		me = (popup.user == None or popup.user == self.frame.np.config.sections["server"]["login"])
 		items = popup.get_children()
 		
 		items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
 		items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
 		items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+		items[9].set_active(self.frame.UserIpIsBlocked(user))
+		items[9].set_sensitive(not me)
+		items[10].set_active(self.frame.UserIpIsIgnored(user))
+		items[10].set_sensitive(not me)
 	
 		return popup
 		
@@ -119,6 +132,9 @@ class PrivateChats(IconNotebook):
 			n = self.page_num(child)
 			page = self.get_nth_page(n)
 			username =  [user for user, tab in self.users.items() if tab.Main is page][0]
+			if event.button == 2:
+				self.users[username].OnClose(widget)
+				return True
 			if event.button == 3:
 				menu = self.TabPopup(username)
 				menu.popup(None, None, None, event.button, event.time)
@@ -131,7 +147,6 @@ class PrivateChats(IconNotebook):
 		if msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
 			return
 
-		
 		self.SendMessage(msg.user, None)
 		chat = self.users[msg.user]
 		self.request_changed(chat.Main)
@@ -147,7 +162,7 @@ class PrivateChats(IconNotebook):
 			self.frame.RequestIcon(self.frame.PrivateChatTabLabel)
 			# Show notifications if the private chats notebook isn't selected,
 			# the tab is not selected, or the main window isn't mapped
-			if self.get_current_page() != self.page_num(chat.Main) or self.frame.MainNotebook.get_current_page() != 1 or not self.frame.is_mapped:
+			if self.get_current_page() != self.page_num(chat.Main) or self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or not self.frame.is_mapped:
 				self.frame.Notification("private", msg.user)
 
 		# SEND CLIENT VERSION to user if the following string is sent
@@ -191,18 +206,50 @@ class PrivateChats(IconNotebook):
 			tab.GetUserStatus(0)
 			
 	def UpdateCompletions(self):
-		for user in self.users.values():
-			user.GetCompletionList()
-			
-class PrivateChat(PrivateChatTab):
-	def __init__(self, chats, user):
-		PrivateChatTab.__init__(self, False)
+		config = self.frame.np.config.sections["words"]
+		clist = [self.frame.np.config.sections["server"]["login"], "nicotine"] + self.users.keys()
+		if config["buddies"]:
+			clist += [i[0] for i in self.frame.userlist.userlist]
+		if config["aliases"]:
+			clist += ["/"+k for k in self.frame.np.config.aliases.keys()]
+		if config["commands"]:
+			clist += self.CMDS
+		if config["roomnames"]:
+			clist += self.frame.chatrooms.roomsctrl.rooms
 		
+		# no duplicates
+		self.clist = list(set(clist))
+		self.clist.sort(key=str.lower)
+		
+		for user in self.users.values():
+			user.GetCompletionList(clist=self.clist)
+			
+class PrivateChat:
+	def __init__(self, chats, user):
 		self.user = user
 		self.chats = chats
 		self.frame = chats.frame
+		
+
+		self.wTree = gtk.glade.XML(os.path.join(os.path.dirname(os.path.realpath(__file__)), "privatechat.glade" ), None, 'nicotine' ) 
+		widgets = self.wTree.get_widget_prefix("")
+		for i in widgets:
+			name = gtk.glade.get_widget_name(i)
+			self.__dict__[name] = i
+			#print name, i
+		self.PrivateChatTab.remove(self.Main)
+		self.PrivateChatTab.destroy()
+		self.wTree.signal_autoconnect(self)
+		#Get glade widgets' tooltips
+		self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
+		#if not self.frame.np.config.sections["ui"]["tooltips"]:
+		self.tooltips.disable()
+		self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
 		if not self.frame.np.config.sections["ui"]["tooltips"]:
 			self.tooltips.disable()
+
+
+
 		self.logfile = None
 		self.autoreplied = 0
 		self.offlinemessage = 0
@@ -213,6 +260,9 @@ class PrivateChat(PrivateChatTab):
 		self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self.Encoding.set_size_request(100, -1)
 		self.Encoding.set_model(self.EncodingStore)
+		cell = gtk.CellRendererText()
+		self.Encoding.pack_start(cell, True)
+		self.Encoding.add_attribute(cell, 'text', 0)
 		cell2 = gtk.CellRendererText()
 		self.Encoding.pack_start(cell2, False)
 		self.Encoding.add_attribute(cell2, 'text', 1)
@@ -259,6 +309,8 @@ class PrivateChat(PrivateChatTab):
 			("$" + _("Add user to list"), popup.OnAddToList),
 			("$" + _("Ban this user"), popup.OnBanUser),
 			("$" + _("Ignore this user"), popup.OnIgnoreUser),
+			("$" + _("B_lock this user's IP Address"), popup.OnBlockUser),
+			("$" + _("Ignore this user's IP Address"), popup.OnIgnoreIP),
 		)
 		popup.set_user(user)
 		self.popup_menu = popup = PopupMenu(self.frame)
@@ -281,7 +333,7 @@ class PrivateChat(PrivateChatTab):
 		self.ChatScroll.connect("key_press_event", self.OnPopupMenu)
 
 		self.UpdateColours()
-		self.GetCompletionList()
+		self.GetCompletionList(clist=self.chats.clist)
 		
 		self.ReadPrivateLog()
 		
@@ -340,10 +392,14 @@ class PrivateChat(PrivateChatTab):
 	
 	def OnPopupMenuUser(self, widget):
 		items = self.popup_menu_user.get_children()
-		
+		me = ( self.popup_menu_user.user == None or self.popup_menu_user.user == self.frame.np.config.sections["server"]["login"])
 		items[6].set_active(self.user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
 		items[7].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
 		items[8].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
+		items[9].set_active(self.frame.UserIpIsBlocked(self.user))
+		items[9].set_sensitive(not me)
+		items[10].set_active(self.frame.UserIpIsIgnored(self.user))
+		items[10].set_sensitive(not me)
 	
 		return True
 	
@@ -374,9 +430,23 @@ class PrivateChat(PrivateChatTab):
 			line = "[%s] %s" % (self.user, self.frame.CensorChat(text))
 			speech = self.frame.CensorChat(text)
 			tag = self.tag_remote
-		line = self.frame.np.decode(line, self.encoding)
 		timestamp_format=self.frame.np.config.sections["logging"]["private_timestamp"]
-		AppendLine(self.ChatScroll, line, tag, timestamp=timestamp, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
+		if status and not self.offlinemessage:
+			AppendLine(self.ChatScroll, _("* Message(s) sent while you were offline. Timestamps are reported by the server and can be off."), self.tag_hilite, timestamp_format=timestamp_format)
+			self.offlinemessage = 1
+		if not status and self.offlinemessage:
+			self.offlinemessage = False
+		line = self.frame.np.decode(line, self.encoding)
+		if status:
+			# The timestamps from the server are off by a lot, so we'll only use them when this is an offline message
+			# Also, they are in UTC so we need to correct them
+			if daylight:
+				timestamp -= (3600*daylight)
+			else:
+				timestamp += altzone
+			AppendLine(self.ChatScroll, line, self.tag_hilite, timestamp=timestamp, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
+		else:
+			AppendLine(self.ChatScroll, line, tag, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
 		if self.Log.get_active():
 			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line)
 		
@@ -384,13 +454,14 @@ class PrivateChat(PrivateChatTab):
 		if self.frame.away and not self.autoreplied and autoreply:
 			self.SendMessage("[Auto-Message] %s" % autoreply)
 			self.autoreplied = 1
-		if status and not self.offlinemessage:
-			
-			AppendLine(self.ChatScroll, _("* Message(s) sent while you were offline."), self.tag_hilite, timestamp_format=timestamp_format)
-			self.offlinemessage = 1
 		self.frame.new_tts(self.frame.np.config.sections["ui"]["speechprivate"] %{"user":self.frame.tts_clean(self.user), "message": self.frame.tts_clean(speech)} )
 
 	def SendMessage(self, text):
+		tuple = self.frame.pluginhandler.OutgoingPrivateChatEvent(self.user, text)
+		if tuple == None:
+			return
+		(u, text) = tuple
+
 		my_username = self.frame.np.config.sections["server"]["login"]
 		if text[:4] == "/me ":
 			line = "* %s %s" % (my_username, text[4:])
@@ -416,8 +487,6 @@ class PrivateChat(PrivateChatTab):
 		else:
 			self.frame.np.queue.put(slskmessages.MessageUser(self.user, self.frame.AutoReplace(text)))
 			
-	CMDS = ["/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
-		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/unban ", "/unignore ", "/clear", "/quit", "/exit", "/rescan", "/nsa", "/info", "/ctcpversion", "/join"]
 
 		
 	def threadAlias(self, alias):
@@ -426,7 +495,7 @@ class PrivateChat(PrivateChatTab):
 			return
 		if text[:2] == "//":
 			text = text[1:]
-		self.frame.np.queue.put(slskmessages.SayChatroom(self.room, self.frame.AutoReplace(text)))
+		self.frame.np.queue.put(slskmessages.MessageUser(self.user, self.frame.AutoReplace(text)))
 
 			
 	def OnEnter(self, widget):
@@ -471,7 +540,10 @@ class PrivateChat(PrivateChatTab):
 				self.frame.OnUserBrowse(None)
 		elif cmd == "/ip":
 			if args:
-				self.frame.np.queue.put(slskmessages.GetPeerAddress(args))
+				user = args
+				if user not in self.frame.np.ip_requested:
+					self.frame.np.ip_requested.append(user)
+				self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
 		elif cmd == "/nsa":
 			if args:
 				self.frame.LocalUserInfoRequest(args)
@@ -517,6 +589,9 @@ class PrivateChat(PrivateChatTab):
 		elif cmd == "/ignore":
 			if args:
 				self.frame.IgnoreUser(args)
+		elif cmd == "/ignoreip":
+			if args:
+				self.frame.IgnoreIP(args)
 		elif cmd == "/unban":
 			if args:
 				self.frame.UnbanUser(args)
@@ -543,6 +618,8 @@ class PrivateChat(PrivateChatTab):
 			self.Attach()
 		elif cmd == "/rescan":
 			self.frame.OnRescan()
+		elif cmd[:1] == "/" and self.frame.pluginhandler.TriggerPrivateCommandEvent(self.user, cmd[1:], args):
+			pass
 		elif cmd and cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
 			self.frame.logMessage(_("Command %s is not recognized") % text)
 			return
@@ -696,7 +773,7 @@ class PrivateChat(PrivateChatTab):
 		color = self.getUserStatusColor(self.status)
 		self.changecolour(self.tag_username, color)
 		statusword = [_("Offline"), _("Away"), _("Online")][status]
-		title = _("Nicotine+ Private Chat: %s (%s)") % (self.user, statusword)
+		title = _("Nicotine+ Private Chat: %(user)s (%(status)s)") % {'user':self.user, 'status':statusword}
 		self.chats.set_detached_tab_title(self.Main, title)
 
 	
@@ -708,7 +785,7 @@ class PrivateChat(PrivateChatTab):
 		self.destroy()
 
 
-	def GetCompletionList(self, ix=0, text=""):
+	def GetCompletionList(self, ix=0, text="", clist=[]):
 		config = self.frame.np.config.sections["words"]
 		completion = self.ChatLine.get_completion()
 		completion.set_popup_single_match(not config["onematch"])
@@ -721,20 +798,6 @@ class PrivateChat(PrivateChatTab):
 		if not config["tab"]:
 			return
 		
-		clist = [self.user, self.frame.np.config.sections["server"]["login"], "nicotine"]
-		if config["buddies"]:
-			clist += [i[0] for i in self.frame.userlist.userlist]
-		if config["aliases"]:
-			clist += ["/"+k for k in self.frame.np.config.aliases.keys()]
-		if config["commands"]:
-			clist += self.CMDS
-		if config["roomnames"]:
-			clist += self.frame.chatrooms.roomsctrl.rooms
-		
-		# no duplicates
-		clist = list(sets.Set(clist))
-		clist.sort(key=str.lower)
-		
 		completion.set_popup_completion(False)
 		if config["dropdown"]:
 			for word in clist:
@@ -764,15 +827,14 @@ class PrivateChat(PrivateChatTab):
 			return False
 		ix = widget.get_position()
 		text = widget.get_text()[:ix].split(" ")[-1]
+		preix = ix - len(text)
 		
-		if widget.get_text()[:1] == "/":
-			self.GetCompletionList(ix, text)
-			
-		completion, single = GetCompletion(text, self.clist)
+		completion, single = GetCompletion(text, self.chats.clist)
 		
 		if completion:
+			widget.delete_text(preix, ix)
 			widget.insert_text(completion, ix)
-			widget.set_position(ix + len(completion))
+			widget.set_position(preix + len(completion))
 		widget.emit_stop_by_name("key_press_event")
 		return True
 
