@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2007 daelstorm. All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,7 +18,7 @@
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
 import gtk
-import gobject
+import gobject, pango
 
 import re
 import sre_constants
@@ -27,13 +28,14 @@ import random
 
 from pynicotine import slskmessages
 
-from nicotine_glade import SearchTab
-from utils import InitialiseColumns, IconNotebook, PopupMenu, FastListModel, Humanize, PressHeader
+from utils import InitialiseColumns, IconNotebook, PopupMenu, FastListModel, Humanize, HumanSpeed, HumanSize, PressHeader
 from dirchooser import ChooseDir
 from entrydialog import *
 from pynicotine.utils import _
 from utils import InputDialog
 
+from time import time
+
 class WishList( gtk.Dialog):
 	def __init__(self, frame):
 		gtk.Dialog.__init__(self)
@@ -167,13 +169,26 @@ class Searches(IconNotebook):
 		self.searchid = int(random.random() * (2**31-1))
 		self.searches = {}
 		self.usersearches = {}
+		self.users = {}
 		self.timer = None
 		self.disconnected = 0
 		ui = self.frame.np.config.sections["ui"]
-		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"], ui["tab_icons"])
+		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"], ui["tab_icons"], ui["tab_reorderable"])
 		self.popup_enable()
 		#frame.SearchEntryCombo.disable_activate()
 		
+
+		self.WishListDialog = WishList(frame)
+		
+		self.UpdateColours()
+		self.show()
+
+	def LoadConfig(self):
+		"""
+		Add search history to SearchEntryCombo later and connect Wishlist, 
+		after widgets have been created by glade. (doing this in __init__ causes tracebacks 
+		during import with gtk.glade.XML in frame.py)
+		"""
 		items = self.frame.np.config.sections["searches"]["history"]
 		templist = []
 		for i in items:
@@ -181,10 +196,8 @@ class Searches(IconNotebook):
 				templist.append(i)
 		for i in templist:
 			self.frame.SearchEntryCombo.append_text(i)
-		self.WishListDialog = WishList(frame)
 		self.frame.WishList.connect("clicked", self.WishListDialog.Toggle)
-		self.UpdateColours()
-		self.show()
+
 	def SetInterval(self, msg):
 		self.interval = msg.seconds
 		if not self.disconnected:
@@ -255,8 +268,30 @@ class Searches(IconNotebook):
 		else:
 			mode = 0
 			
-		self.DoSearch(text, mode, users, room)
-		self.frame.SearchEntry.set_text("")
+		feedback = None
+		if mode == 0:
+			feedback = self.frame.pluginhandler.OutgoingGlobalSearchEvent(text)
+			if feedback != None:
+				text = feedback[0]
+		elif mode == 1:
+			feedback = self.frame.pluginhandler.OutgoingRoomSearchEvent(room, text)
+			if feedback != None:
+				(room, text) = feedback
+		elif mode == 2:
+			feedback = self.frame.pluginhandler.OutgoingBuddySearchEvent(text)
+			if feedback != None:
+				text = feedback[0]
+		elif mode == 3:
+			feedback = self.frame.pluginhandler.OutgoingUserSearchEvent(users)
+			if feedback != None:
+				users = feedback[0]
+		else:
+			print "Unknown search mode, not using plugin system. Fix me!"
+			feedback = True
+			
+		if feedback != None:
+			self.DoSearch(text, mode, users, room)
+			self.frame.SearchEntry.set_text("")
 		
 	def NewWish(self, wish):
 		if wish in self.frame.np.config.sections["server"]["autosearch"]:
@@ -297,7 +332,7 @@ class Searches(IconNotebook):
 		elif mode == 2:
 			self.DoBuddiesSearch(self.searchid, text)
 		elif mode == 3 and users != [] and users[0] != '':
-				self.DoPeerSearch(self.searchid, text, users)
+			self.DoPeerSearch(self.searchid, text, users)
 				
 		self.searchid += 1
 		
@@ -400,7 +435,8 @@ class Searches(IconNotebook):
 				continue
 			id[2].ChangeColours()
 		self.frame.SetTextBG(self.WishListDialog.WishlistView)
-			
+		
+
 	def saveColumns(self):
 		page_num = self.get_current_page()
 		if page_num is not None:
@@ -435,7 +471,7 @@ class Searches(IconNotebook):
 		return popup
 		
 	def on_tab_click(self, widget, event, child):
-		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+		if event.type == gtk.gdk.BUTTON_PRESS:
 			id = None
 			n = self.page_num(child)
 			page = self.get_nth_page(n)
@@ -445,51 +481,97 @@ class Searches(IconNotebook):
 				if data[2].Main is page:
 					id = search
 					break
-			if id is not None:
+			if id is None:
+				print "ID is none"
+				return
+			if event.button == 2:
+				self.searches[id][2].OnClose(widget)
+				return True
+			if event.button == 3:
 				menu = self.TabPopup(id)
 				menu.popup(None, None, None, event.button, event.time)
 				return True
-			return False
 		return False
 	
-class Search(SearchTab):
+class Search:
+	WAIT_BEFORE_DISPLAYING = 5000 # in milliseconds
 	def __init__(self, Searches, text, id, mode, remember):
-		SearchTab.__init__(self, False)
-
-#		self.ResultsList.set_double_buffered(False)
-
 		self.Searches = Searches
 		self.frame = Searches.frame
+		self.tooltips = self.frame.tooltips
+		if not self.frame.np.config.sections["ui"]["tooltips"]:
+			self.tooltips.disable()
+
+		self.wTree = gtk.glade.XML(os.path.join(os.path.dirname(os.path.realpath(__file__)), "search.glade" ), None, 'nicotine' ) 
+		widgets = self.wTree.get_widget_prefix("")
+		for i in widgets:
+			name = gtk.glade.get_widget_name(i)
+			self.__dict__[name] = i
+		self.SearchTab.remove(self.Main)
+		self.SearchTab.destroy()
+		self.wTree.signal_autoconnect(self)
+
+		self.FilterBitrate_List = gtk.ListStore(gobject.TYPE_STRING)
+		self.FilterBitrate.set_model(self.FilterBitrate_List)
+		self.FilterBitrate.set_text_column(0)
+		self.FilterSize_List = gtk.ListStore(gobject.TYPE_STRING)
+		self.FilterSize.set_model(self.FilterSize_List)
+		self.FilterSize.set_text_column(0)
+		self.FilterCountry_List = gtk.ListStore(gobject.TYPE_STRING)
+		self.FilterCountry.set_model(self.FilterCountry_List)
+		self.FilterCountry.set_text_column(0)
+		self.FilterIn_List = gtk.ListStore(gobject.TYPE_STRING)
+		self.FilterIn.set_model(self.FilterIn_List)
+		self.FilterIn.set_text_column(0)
+		self.FilterOut_List = gtk.ListStore(gobject.TYPE_STRING)
+		self.FilterOut.set_model(self.FilterOut_List)
+		self.FilterOut.set_text_column(0)
+		self.wTree.signal_autoconnect(self)
+#		self.ResultsList.set_double_buffered(False)
+
+
 		self.text = text
 		self.id = id
 		self.mode = mode
 		self.remember = remember
-		self.users = []
 		self.usersiters = {}
+		self.users = []
 		self.resultslimit = 2000
 		self.QueryLabel.set_text(text)
 
 		self.all_data = []
 		self.filters = None
-		self.COLUMN_TYPES = [int, str, str, str, str, str, str, str, str, str,
-			int, str, str, long, int, int, int]
+		# num, user, filename, h_size, h_speed, h_queue, immediatedl, h_bitrate, length, self.get_flag(user, country), directory, bitrate, fullpath, country,  size, speed, queue, status]
+		self.COLUMN_TYPES = [int, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str,
+			int, str, str, gobject.TYPE_ULONG, int, int, int]
 		self.resultsmodel = gtk.TreeStore(* self.COLUMN_TYPES )
 
-		if self.frame.np.config.sections["searches"]["enablefilters"]:
-			filter = self.frame.np.config.sections["searches"]["defilter"]
-			self.FilterIn.child.set_text(filter[0])
-			self.FilterOut.child.set_text(filter[1])
-			self.FilterSize.child.set_text(filter[2])
-			self.FilterBitrate.child.set_text(filter[3])
-			self.FilterFreeSlot.set_active(filter[4])
-			if(len(filter) > 5):
-				self.FilterCountry.child.set_text(filter[5])
-			self.filtersCheck.set_active(1)
+		
 
 		if mode > 0:
 			self.RememberCheckButton.set_sensitive(False)
 		self.RememberCheckButton.set_active(remember)
-
+		
+		self.PopulateFilters()
+		
+		self.FilterSize.clear()
+		sizecell = gtk.CellRendererText()
+		sizecell.set_property("xalign", 1)
+		self.FilterSize.pack_start(sizecell, True)
+		self.FilterSize.add_attribute(sizecell, "text", 0)
+		
+		self.FilterBitrate.clear()
+		bit_cell = gtk.CellRendererText()
+		bit_cell.set_property("xalign", 1)
+		self.FilterBitrate.pack_start(bit_cell, True)
+		self.FilterBitrate.add_attribute(bit_cell, "text", 0)
+		
+		self.FilterIn.connect("changed", self.OnFilterChanged)
+		self.FilterOut.connect("changed", self.OnFilterChanged)
+		self.FilterSize.connect("changed", self.OnFilterChanged)
+		self.FilterBitrate.connect("changed", self.OnFilterChanged)
+		self.FilterCountry.connect("changed", self.OnFilterChanged)
+		
 		self.selected_results = []
 		self.selected_users = []
 
@@ -497,17 +579,20 @@ class Search(SearchTab):
 		self.ResultsList.set_property("show-expanders", False)
 		self.ResultsList.set_property("rules-hint", True)
 		cols = InitialiseColumns(self.ResultsList,
-			[_("Number"), 25, "text", self.CellDataFunc],
+			[_("Number"), 50, "text", self.CellDataFunc],
 			[_("User"), 100, "text", self.CellDataFunc],
 			[_("Filename"), 250, "text", self.CellDataFunc],
 			[_("Size"), 100, "text", self.CellDataFunc],
-			[_("Speed"), 50, "text", self.CellDataFunc],
+			[_("Speed"), 90, "text", self.CellDataFunc],
 			[_("In queue"), 50, "text", self.CellDataFunc],
 			[_("Immediate Download"), 20, "text", self.CellDataFunc],
 			[_("Bitrate"), 50, "text", self.CellDataFunc],
 			[_("Length"), 50, "text", self.CellDataFunc],
+			[_("Country"), 25, "pixbuf"],
 			[_("Directory"), 1000, "text", self.CellDataFunc],
+			
 		)
+		self.col_num, self.col_user, self.col_file, self.col_size, self.col_speed, self.col_queue, self.col_immediate, self.col_bitrate, self.col_length, self.col_country, self.col_directory = cols
 		cols[0].get_widget().hide()
 		for i in range (10):
 			
@@ -517,21 +602,26 @@ class Search(SearchTab):
 			# Read Show / Hide column settings from last session
 			cols[i].set_visible(self.frame.np.config.sections["columns"]["search"][i])
 		self.ResultsList.set_model(self.resultsmodel)
-		for ix in range(len(cols)):
-			col = cols[ix]
+		#for ix in range(len(cols)):
+			#col = cols[ix]
 			#col.connect("clicked", self.OnResort, ix)
 			#for r in col.get_cell_renderers():
 				#r.set_fixed_height_from_font(1)
-		cols[0].set_sort_column_id(0)
-		cols[1].set_sort_column_id(1)
-		cols[2].set_sort_column_id(2)
-		cols[3].set_sort_column_id(13)
-		cols[4].set_sort_column_id(14)
-		cols[5].set_sort_column_id(15)
-		cols[6].set_sort_column_id(6)
-		cols[7].set_sort_column_id(10)
-		cols[8].set_sort_column_id(8)
-		cols[9].set_sort_column_id(9)
+		for r in self.col_num.get_cell_renderers() + self.col_size.get_cell_renderers() +  self.col_length.get_cell_renderers():
+			r.set_property("xalign",1)
+
+		self.col_num.set_sort_column_id(0)
+		self.col_user.set_sort_column_id(1)
+		self.col_file.set_sort_column_id(2)
+		self.col_size.set_sort_column_id(14)
+		
+		self.col_speed.set_sort_column_id(15)
+		self.col_queue.set_sort_column_id(16)
+		self.col_immediate.set_sort_column_id(6)
+		self.col_bitrate.set_sort_column_id(11)
+		self.col_length.set_sort_column_id(8)
+		self.col_country.set_sort_column_id(13)
+		self.col_directory.set_sort_column_id(10)
 		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
 			self.ResultsList.set_enable_tree_lines(True)
 		
@@ -558,7 +648,61 @@ class Search(SearchTab):
 		self._more_results = 0
 		self.new_results = []
 		self.ChangeColours()
-
+		
+	def OnFilterChanged(self, widget):
+		model = widget.get_model()
+		iter = widget.get_active_iter()
+		if iter:
+			self.OnRefilter(None)
+			
+	def PopulateFilters(self):
+		if self.frame.np.config.sections["searches"]["enablefilters"]:
+			filter = self.frame.np.config.sections["searches"]["defilter"]
+			self.FilterIn.child.set_text(filter[0])
+			self.FilterOut.child.set_text(filter[1])
+			self.FilterSize.child.set_text(filter[2])
+			self.FilterBitrate.child.set_text(filter[3])
+			self.FilterFreeSlot.set_active(filter[4])
+			if(len(filter) > 5):
+				self.FilterCountry.child.set_text(filter[5])
+			self.filtersCheck.set_active(1)
+			
+		for i in [0, 128, 160, 192, 256, 320]:
+			self.FilterBitrate.get_model().append([i])
+		for i in [">10MiB", "<10MiB", "<5MiB", "<1MiB", ">0"]:
+			self.FilterSize.get_model().append([i])
+		s_config = self.frame.np.config.sections["searches"]
+		
+		for i in s_config["filterin"]:
+			self.AddCombo(self.FilterIn, i, True)
+		for i in s_config["filterout"]:
+			self.AddCombo(self.FilterOut, i, True)
+		for i in s_config["filtersize"]:
+			self.AddCombo(self.FilterSize, i, True)
+		for i in s_config["filterbr"]:
+			self.AddCombo(self.FilterBitrate, i, True)
+		for i in s_config["filtercc"]:
+			self.AddCombo(self.FilterCountry ,i, True)
+			
+	def AddCombo(self, ComboboxEntry, text, list=False):
+		text = text.strip()
+		if not text:
+			return False
+		model = ComboboxEntry.get_model()
+		iter = model.get_iter_root()
+		match = False
+		while iter is not None:
+			value = model.get_value(iter, 0)
+			if value.strip() == text:
+				match = True
+			iter = model.iter_next(iter)
+		if not match:
+			if list:
+				model.append([text])
+			else:
+				model.prepend([text])
+	
+		
 	def Attach(self, widget=None):
 		self.Searches.attach_tab(self.Main)
 
@@ -570,17 +714,22 @@ class Search(SearchTab):
 	def AddResult(self, msg, user, country):
 		if user in self.users:
 			return
-		results = []
-		
+		if user not in self.Searches.users.keys():
+			if user in self.frame.np.users.keys():
+				self.Searches.users[user] = self.frame.np.users[user].status
+			else:
+				self.Searches.users[user] = 0
+				self.frame.np.queue.put(slskmessages.AddUser(user))
+			if user == self.frame.np.config.sections["server"]["login"]:
+				self.Searches.users[user] = 1
+				
 		self.users.append(user)
+		results = []
 
-		self.frame.np.queue.put(slskmessages.AddUser(user))
-		
 		if msg.freeulslots:
 			imdl = _("Y")
 		else:
 			imdl = _("N")
-		ix = len(self.all_data) + 1
 		decode = self.frame.np.decode
 		for result in msg.list:
 			name = result[1].split('\\')[-1]
@@ -595,81 +744,100 @@ class Search(SearchTab):
 				bitrate = str(a[0]) + bitrate
 				br = a[0]
 				length = '%i:%02i' %(a[1] / 60, a[1] % 60)
-			results.append([user, name, result[2], msg.ulspeed, msg.inqueue, imdl, bitrate, length, dir, br, result[1], country, 1])
-			ix += 1
-			
+			results.append([user, name, result[2], msg.ulspeed, msg.inqueue, imdl, bitrate, length, dir, br, result[1], country, self.Searches.users[user]])
 		if results:
 			self.new_results += results
 			
-			if self._more_results == 0:
+			if self._more_results == 0 and len(self.resultsmodel) < self.frame.np.config.sections['searches']["max_displayed_results"]:
 				self._more_results = 1
-				gobject.timeout_add(1000, self._realaddresults)
-			else:
-				self._more_results = 2
+				gobject.timeout_add(self.WAIT_BEFORE_DISPLAYING, self._realaddresults)
 			return len(results)
 	
 	def _realaddresults(self):
-		if self._more_results == 2:
-			self._more_results = 1
-			return True
-		
+		if "_more_results" not in self.__dict__:
+			return
+		self._more_results = 0
 		r = self.new_results
 		self.new_results = []
-		self._more_results = 0
 
 		res = self.append(r)
 
 		if res:
 			self.frame.Searches.request_changed(self.Main)
-			if self.frame.MainNotebook.get_current_page() != 4:
-				self.frame.SearchTabLabel.set_image(self.frame.images["online"])
+			if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.searchvbox):
+				self.frame.SearchTabLabel.child.set_image(self.frame.images["online"])
 			#self.frame.RequestIcon(self.frame.SearchTabLabel)
 
 		rows = len(self.all_data)
 		for c in self.ResultsList.get_columns():
 			for r in c.get_cell_renderers():
-				r.set_fixed_height_from_font(1)
+				if type(r) is not gtk.CellRendererPixbuf:
+					r.set_fixed_height_from_font(1)
 
 		return False
 		
+	def get_flag(self, user, flag=None):
+		#flag = users[user].country
+		if flag is not None:
+			flag = "flag_"+flag
+			self.frame.flag_users[user] = flag
+		else:
+			flag = self.frame.GetUserFlag(user)
+
+		return self.frame.GetFlagImage(flag)
 		
 	def append(self, results):
-		ix = len(self.all_data) + 1
-		#l = len(self.data)
+		itercounter = len(self.all_data) + 1
+		displaycounter = len(self.resultsmodel)
 
 		encode = self.frame.np.encodeuser
 		
 		returned = 0
-		
+		if itercounter > self.frame.np.config.sections['searches']["max_stored_results"]:
+			return returned
 		for r in results:
 
 			user, filename, size, speed, queue, immediatedl, h_bitrate, length, directory, bitrate, fullpath,  country, status = r
-			
-			h_size = Humanize(long(size))
-			h_speed = Humanize(speed)
+			if user in self.Searches.users.keys() and status != self.Searches.users[user]:
+				status = self.Searches.users[user]
+			if status is None:
+				status = 0
+			h_size = HumanSize(long(size))
+			h_speed = HumanSpeed(speed)
 			h_queue = Humanize(queue)
 			if self.usersGroup.get_active() and user not in self.usersiters:
-				self.usersiters[user] = self.resultsmodel.append(None, [0, user, "", "", h_speed, h_queue, immediatedl, "", "", "", 0, "", "", 0, speed, queue, status])
-			row = [ix, user, filename, h_size, h_speed, h_queue, immediatedl, h_bitrate, length, directory,  bitrate, fullpath, country,  size, speed, queue, status]
+				self.usersiters[user] = self.resultsmodel.append(None, [0, user, "", "", h_speed, h_queue, immediatedl, "", "", self.get_flag(user, country), "", 0, "", country, 0, speed, queue, status])
+			row = [itercounter, user, filename, h_size, h_speed, h_queue, immediatedl, h_bitrate, length,
+			       directory,  bitrate, fullpath, country,  size, speed, queue, status]
 
 			self.all_data.append(row)
-			if not self.filters or self.check_filter(row):
-				encoded_row = [ix, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl, h_bitrate, length, encode(directory, user), bitrate, encode(fullpath, user), country,  size, speed, queue, status]
-
+			if (displaycounter + returned < self.frame.np.config.sections['searches']["max_displayed_results"]) and (not self.filters or self.check_filter(row)):
+				encoded_row = [itercounter, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl,
+				               h_bitrate, length, self.get_flag(user, country), encode(directory, user), bitrate,
+				               encode(fullpath, user), country,  size, speed, queue, status]
+				#print user, status
 				if user in self.usersiters:
 					iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
 				else:
-					iter = self.resultsmodel.append(None, encoded_row)
+					try:
+						iter = self.resultsmodel.append(None, encoded_row)
+					except Exception, e:
+						print "Search row error", e, encoded_row
+						print 
+						iter=None
+
 				path = self.resultsmodel.get_path(iter)
 				if path is not None:
 					if self.usersGroup.get_active() and self.ExpandButton.get_active():
 						self.ResultsList.expand_to_path(path)
 				returned += 1
-			ix += 1
-		
+			itercounter += 1
+			if itercounter > self.frame.np.config.sections['searches']["max_stored_results"]:
+				break
 		return returned
 			
 	def updateStatus(self, user, status):
+		self.Searches.users[user] = status
 		pos = 0
 		for r in self.all_data:
 		
@@ -682,13 +850,13 @@ class Search(SearchTab):
 			selected_user = self.resultsmodel.get_value(iter, 1)
 			
 			if selected_user == user:
-				self.resultsmodel.set_value(iter, 16, status)
+				self.resultsmodel.set_value(iter, 17, status)
 			if self.resultsmodel.iter_has_child(iter):
 				child = self.resultsmodel.iter_children(iter)
 				while child is not None:
 					selected_user = self.resultsmodel.get_value(child, 1)
 					if selected_user == user:
-						self.resultsmodel.set_value(child, 16, status)
+						self.resultsmodel.set_value(child, 17, status)
 					child = self.resultsmodel.iter_next(child)
 			iter = self.resultsmodel.iter_next(iter)
 		
@@ -696,11 +864,11 @@ class Search(SearchTab):
 		col = self.sort_col
 		order = self.sort_order
 		if col == 3:
-			col = 13
-		elif col == 4:
 			col = 14
-		elif col == 5:
+		elif col == 4:
 			col = 15
+		elif col == 5:
+			col = 16
 
 		if self.COLUMN_TYPES[col] == gobject.TYPE_STRING:
 			compare = locale.strcoll
@@ -724,24 +892,30 @@ class Search(SearchTab):
 
 		factor = 1
 		if factorize:
-			if filter.lower()[-1] == "g":
-				factor = 1024*1024*1024
+			base = 1024
+			if filter[-1:].lower() == 'b':
+				filter = filter[:-1] # stripping off the b, we always assume it means bytes
+			if filter[-1:].lower() == 'i':
+				base = 1000
 				filter = filter[:-1]
-			elif filter.lower()[-1] == "m":
-				factor = 1024*1024
+			if filter.lower()[-1:] == "g":
+				factor = pow(base, 3)
 				filter = filter[:-1]
-			elif filter.lower()[-1] == "k":
-				factor = 1024
+			elif filter.lower()[-1:] == "m":
+				factor = pow(base, 2)
+				filter = filter[:-1]
+			elif filter.lower()[-1:] == "k":
+				factor = base
 				filter = filter[:-1]
 
 		if not filter:
 			return True
 
-		if not filter.isdigit():
+		try: 
+			filter = long(filter) * factor
+		except TypeError:
 			return True
 
-		filter = long(filter) * factor
-
 		if eval(str(value)+op+str(filter), {}):
 			return True
 
@@ -749,6 +923,8 @@ class Search(SearchTab):
 
 	def check_filter(self, row):
 		filters = self.filters
+		if not self.filtersCheck.get_active():
+			return True
 		if filters[0] and not filters[0].search(row[2].lower()):
 			return False
 		if filters[1] and filters[1].search(row[2].lower()):
@@ -763,19 +939,19 @@ class Search(SearchTab):
 			for cc in filters[5]:
 				if not cc:
 					continue
+				if row[12] is None:
+					return False
+
 				if cc[0] == "-":
-					if row[12] == cc[1:]:
+					if row[12].upper()  == cc[1:].upper():
 						return False
-				elif cc != row[12]:
+				elif cc.upper() != row[12].upper():
 					return False
 		return True
 	
 	def set_filters(self, enable, f_in, f_out, size, bitrate, freeslot, country):
 		
-		if not enable:
-			self.filters = None
-			#self.data = self.all_data[:]
-			return
+
 		if self.frame.np.transfers is None:
 			encode = self.frame.np.encode
 		else:
@@ -814,17 +990,24 @@ class Search(SearchTab):
 		
 		#data = []
 		
+		displaycounter = 0
 		for row in self.all_data:
 			if self.check_filter(row):
-				ix, user, filename,  h_size, h_speed, h_queue, immediatedl, h_bitrate, length, directory,  bitrate, fullpath, country,  size, speed, queue, status = row
+				ix, user, filename,  h_size, h_speed, h_queue, immediatedl, h_bitrate, length, directory,  bitrate, fullpath, country, size, speed, queue, status = row
+				if user in self.Searches.users.keys() and status != self.Searches.users[user]:
+					status = self.Searches.users[user]
+				#user, filename, size, speed, queue, immediatedl, h_bitrate, length, directory, bitrate, fullpath,  country, status
 				if  self.usersGroup.get_active() and user not in self.usersiters:
-					self.usersiters[user] = self.resultsmodel.append(None, [0, user, "", "", h_speed, h_queue, immediatedl, "", "", "", 0, "", "", 0, speed, queue, status])
-				encoded_row = [ix, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl, h_bitrate, length, encode(directory, user), bitrate, encode( fullpath, user), country,  size, speed, queue, status]
+					self.usersiters[user] = self.resultsmodel.append(None, [0, user, "", "", h_speed, h_queue, immediatedl, "", "", self.get_flag(user, country), "", 0, "", country, 0, speed, queue, status])
+				encoded_row = [ix, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl, h_bitrate, length, self.get_flag(user, country), encode(directory, user), bitrate, encode( fullpath, user), country,  size, speed, queue, status]
 
 				if user in self.usersiters:
 					iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
 				else:
 					iter = self.resultsmodel.append(None, encoded_row)
+				displaycounter += 1
+			if displaycounter >= self.frame.np.config.sections['searches']["max_displayed_results"]:
+				break
 		
 
 	def OnPopupMenuUsers(self, widget):
@@ -922,6 +1105,9 @@ class Search(SearchTab):
 		self.frame.SetTextBG(self.FilterCountry.child)
 		self.frame.SetTextBG(self.RememberCheckButton)
 		self.frame.SetTextBG(self.FilterFreeSlot)
+		font = self.frame.np.config.sections["ui"]["searchfont"]
+
+		self.frame.ChangeListFont(self.ResultsList, font)
 		
 	def GetUserStatus(self, msg):
 		if msg.user not in self.users:
@@ -944,7 +1130,7 @@ class Search(SearchTab):
 		num = model.get_value(iter, 0)
 		user = model.get_value(iter, 1)
 		fn = None
-  		for r in self.all_data:
+		for r in self.all_data:
 			if num != r[0] or user != r[1]:
 				continue
 			#user = r[1]
@@ -997,12 +1183,10 @@ class Search(SearchTab):
 
 	def CellDataFunc(self, column, cellrenderer, model, iter):
 
-		status = model.get_value(iter, 16)
+		status = model.get_value(iter, 17)
 		imdl = model.get_value(iter, 6)
-		colour = imdl == _("Y") and "search" or "searchq"
-
-		colour = self.frame.np.config.sections["ui"][colour] or None
-
+		color = imdl == _("Y") and "search" or "searchq"
+		colour = None
 		if status == 0:
 			colour = self.frame.np.config.sections["ui"]["searchoffline"]
 			cellrenderer.set_property("background", None)
@@ -1010,6 +1194,7 @@ class Search(SearchTab):
 			colour = "#ffffff"
 			cellrenderer.set_property("background", "#ff0000")
 		else:
+			colour = self.frame.np.config.sections["ui"][color] or None
 			cellrenderer.set_property("background", None)
 		cellrenderer.set_property("foreground", colour)
 
@@ -1032,10 +1217,10 @@ class Search(SearchTab):
 		immediate = model.get_value(iter, 6)
 		bitratestr = model.get_value(iter, 7)
 		length = model.get_value(iter, 8)
-		directory = model.get_value(iter, 9)
+		directory = model.get_value(iter, 10)
 		#bitrate = model.get_value(iter, 10)
-		fn = model.get_value(iter, 11)
-		country = model.get_value(iter, 12)
+		fn = model.get_value(iter, 12)
+		country = model.get_value(iter, 13)
 		data[len(data)] = {"user":user, "fn": fn, "position":num, "filename":filename, "directory":directory, "size":size, "speed":speed, "queue":queue, "immediate":immediate, "bitrate":bitratestr, "length":length, "country":country}
 
 			
@@ -1149,12 +1334,8 @@ class Search(SearchTab):
 				self.ResultsList.collapse_all()
 
 	def OnIgnore(self, widget):
-		#self.RememberCheckButton.set_active(0)
-		#self.RememberCheckButton.set_sensitive(0)
-		#self.OnToggleRemember(self.RememberCheckButton)
 		if self.id in self.Searches.searches.keys():
 			del self.Searches.searches[self.id]
-		
 		widget.set_sensitive(False)
 		
 	def OnClear(self, widget):
@@ -1163,11 +1344,8 @@ class Search(SearchTab):
 		self.resultsmodel.clear()
 
 	def OnClose(self, widget):
-		self.Searches.RemoveTab(self)
-
-	def OnCloseIgnore(self, widget):
-
-		self.OnIgnore(self.button2)
+		if not self.frame.np.config.sections["searches"]["reopen_tabs"]:
+			self.OnIgnore(widget)
 		self.Searches.RemoveTab(self)
 
 	def OnToggleRemember(self, widget):
@@ -1181,9 +1359,12 @@ class Search(SearchTab):
 		text = widget.child.get_text()
 		if not text.strip():
         		return None
+		text = text.strip()
 		history = self.frame.np.config.sections["searches"][title]
 		self.frame.np.config.pushHistory(history, text, 5)
-		widget.append_text( text)
+		
+		self.AddCombo(widget, text)
+	
 		widget.child.set_text(text)
 		return text
 		
