@@ -16,29 +16,38 @@
 # Original copyright below
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
+from __future__ import division
+
 import gtk
 import os, sys
 import urllib
 import gobject
 import gc
 from userinfo import UserTabs
-from nicotine_glade import UserBrowseTab
 
 from utils import InitialiseColumns, PopupMenu, EncodingsMenu, SaveEncoding, Humanize, HumanizeBytes, PressHeader
 from dirchooser import ChooseDir
 from entrydialog import *
 from pynicotine import slskmessages
+from thread import start_new_thread
 
-from pynicotine.utils import _, displayTraceback
+from pynicotine.utils import _, displayTraceback, executeCommand
 
-class UserBrowse(UserBrowseTab):
+class UserBrowse:
 	def __init__(self, userbrowses, user, conn):
-
-		UserBrowseTab.__init__(self, False)
-		
+		self.wTree = gtk.glade.XML(os.path.join(os.path.dirname(os.path.realpath(__file__)), "userbrowse.glade" ), None, "nicotine" ) 
+		widgets = self.wTree.get_widget_prefix("")
+		for i in widgets:
+			name = gtk.glade.get_widget_name(i)
+			self.__dict__[name] = i
+		self.UserBrowseTab.remove(self.Main)
+		self.UserBrowseTab.destroy()
+		#UserBrowseTab.__init__(self, False)
+		self.wTree.signal_autoconnect(self)
 		self.userbrowses = userbrowses
 
 		self.frame = userbrowses.frame
+		self.tooltips = self.frame.tooltips
 		if not self.frame.np.config.sections["ui"]["tooltips"]:
 			self.tooltips.disable()
 		self.user = user
@@ -61,6 +70,9 @@ class UserBrowse(UserBrowseTab):
 		self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self.Encoding.set_size_request(100, -1)
 		self.Encoding.set_model(self.EncodingStore)
+		cell = gtk.CellRendererText()
+		self.Encoding.pack_start(cell, True)
+		self.Encoding.add_attribute(cell, 'text', 0)
 		cell2 = gtk.CellRendererText()
 		self.Encoding.pack_start(cell2, False)
 	
@@ -79,7 +91,6 @@ class UserBrowse(UserBrowseTab):
 		# GTK 2.10
 		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
 			self.FolderTreeView.set_enable_tree_lines(True)
-		
 
 		cols = InitialiseColumns(self.FolderTreeView,
 			[_("Directories"), -1, "text", self.CellDataFunc], #0
@@ -142,8 +153,6 @@ class UserBrowse(UserBrowseTab):
 				("", None),
 				("#" + _("Copy _URL"), self.OnCopyDirURL, gtk.STOCK_COPY),
 				("#" + _("Open in File Manager"), self.OnFileManager, gtk.STOCK_OPEN),
-
-	
 			)
 		else:
 			self.folder_popup_menu.setup(
@@ -177,7 +186,6 @@ class UserBrowse(UserBrowseTab):
 			if parent:
 				parent.connect('button_press_event', PressHeader)
 			# Read Show / Hide column settings from last session
-			
 		self.FileTreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 
 		self.FileTreeView.set_headers_clickable(True)
@@ -206,7 +214,6 @@ class UserBrowse(UserBrowseTab):
 				(2, _("Download"), self.popup_menu_downloads_files, self.OnPopupMenuDummy, gtk.STOCK_GO_DOWN),
 				("", None),
 				("#" + _("Copy _URL"), self.OnCopyURL, gtk.STOCK_COPY),
-				
 			)
 		self.FileTreeView.connect("button_press_event", self.OnFileClicked)
 		self.ChangeColours()
@@ -215,7 +222,7 @@ class UserBrowse(UserBrowseTab):
 			if type(object) is PopupMenu:
 				object.set_user(self.user)
 				
-	def OnPopupMenuDummy(self, widget):
+	def OnPopupMenuDummy(self, widget, something):
 		pass
 	
 	def Attach(self, widget=None):
@@ -248,7 +255,6 @@ class UserBrowse(UserBrowseTab):
 		
 		for i in range(3, 8):
 			items[i].set_sensitive(act)
-
 		return True
 		
 	def ChangeColours(self):
@@ -256,6 +262,10 @@ class UserBrowse(UserBrowseTab):
 		self.frame.SetTextBG(self.FolderTreeView)
 		self.frame.SetTextBG(self.entry4)
 		
+		self.frame.ChangeListFont(self.FolderTreeView, self.frame.np.config.sections["ui"]["browserfont"])
+		self.frame.ChangeListFont(self.FileTreeView, self.frame.np.config.sections["ui"]["browserfont"])
+
+		
 	def CellDataFunc(self, column, cellrenderer, model, iter):
 		colour = self.frame.np.config.sections["ui"]["search"]
 		if colour == "":
@@ -280,7 +290,6 @@ class UserBrowse(UserBrowseTab):
 			else:
 				self.SetDirectory(None)
 			
-			
 	def OnFolderClicked(self, widget, event):
 		if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
 			self.OnDownloadDirectory(widget)
@@ -378,45 +387,40 @@ class UserBrowse(UserBrowseTab):
 		self.directories.clear()
 		directory = ""
 		self.totalsize = 0
+		dirseparator = '\\'
 		if sorted == []:
 			self.AmountShared.set_text(_("Shared: %s") % HumanizeBytes(self.totalsize))
 			return directory
 		for item in sorted:
-			s = item.split("\\")
+			s = item.split(dirseparator)
 			path = ''
 
 			parent = s[0]
 			if parent == '':
-				parent += "\\"
+				parent += dirseparator
 				if parent not in self.directories.keys():
 					self.directories[parent] =  self.DirStore.append(None, [self.decode(parent), parent])
 			parent = s[0]
 			for seq in s[1:]:
-				
-
-				path = "\\".join([parent, seq])
+				path = dirseparator.join([parent, seq])
 				if parent == "":
-					parent = "\\"
+					parent = dirseparator
 				if parent not in self.directories.keys():
 					self.directories[parent] =  self.DirStore.append(None, [self.decode(parent), parent])
-
-
 				if path not in children:
 					children.append(path)
-					self.directories[path] = self.DirStore.append(self.directories[parent], [self.decode(path.split("\\")[-1]), path ] )
+					self.directories[path] = self.DirStore.append(self.directories[parent], [self.decode(path.split(dirseparator)[-1]), path ] )
 				parent = path
 		sortlist = list(self.directories.keys())
 		sortlist.sort()
 		for files in self.shares.values():
 			for filedata in files:
-			
 				self.totalsize += filedata[2]
 		self.AmountShared.set_text(_("Shared: %s") % HumanizeBytes(self.totalsize))
 	
 		directory = sortlist[0]
 		return directory
 	
-			
 	def SetDirectory(self, directory):
 		self.selected_folder = directory
 		self.FileStore.clear()
@@ -425,7 +429,6 @@ class UserBrowse(UserBrowseTab):
 			return
 		
 		path = self.DirStore.get_path( self.directories[directory] )
-		
 
 		files = self.shares[directory]
 
@@ -437,8 +440,6 @@ class UserBrowse(UserBrowseTab):
 			f = [self.decode(file[1]), Humanize(size)]
 			if file[3] == "":
 				f += ["", ""]
-			
-			
 			else:
 				#file[4] is for file types such as 'mp3'
 				attrs = file[4]
@@ -450,13 +451,11 @@ class UserBrowse(UserBrowseTab):
 						try: rl = int(attrs[1])
 						except: rl = 0
 						l = "%i:%02i" % (rl / 60, rl % 60)
-						
 						f += [br, l]
 					else:
 						f += ["", ""]
 				else:
 					f += ["", ""]
-			
 			f += [long(size), rl, file[1]]
 
 			try:
@@ -464,7 +463,6 @@ class UserBrowse(UserBrowseTab):
 			except Exception, error:
 				displayTraceback()
 
-			
 	def OnSave(self, widget):
 		configdir, config = os.path.split(self.frame.np.config.filename)
 		sharesdir = os.path.abspath(configdir+os.sep+"usershares"+os.sep)
@@ -515,17 +513,14 @@ class UserBrowse(UserBrowseTab):
 
 	def OnSelectDir(self, selection):
 		model, iter = selection.get_selected()
-		
 		if iter is None:
 			self.selected_folder = None
 			return
-
 		path = model.get_path(iter)
 		directory = self.DirStore.get_value(self.DirStore.get_iter(path), 1)
 		self.FolderTreeView.expand_to_path(path)
 		self.SetDirectory(directory)
 		
-		
 	def OnResort(self, column, column_id):
 		model = self.FileTreeView.get_model()
 		if model.sort_col == column_id:
@@ -563,7 +558,6 @@ class UserBrowse(UserBrowseTab):
 		go_ahead=0
 		if len(files) > 100:
 			FolderDownload(self.frame, title=_('Nicotine+')+': Download %(num)i files?' % {'num':numfiles}, message=_("Are you sure you wish to download %(num)i files from %(user)s's directory %(folder)s?" %{ 'num':numfiles, 'user':self.user, 'folder':dir } ), data=files, callback=self.folder_download_response )
-			
 		else:
 			# Good to go, we download these
 			for item in files:
@@ -599,15 +593,13 @@ class UserBrowse(UserBrowseTab):
 						bitrate += _(" (vbr)")
 					try: rl = int(attrs[1])
 					except: rl = 0
-					length = "%i:%02i" % (rl / 60, rl % 60)
+					length = "%i:%02i" % (rl // 60, rl % 60)
 
 				files.append(["\\".join([dir, file[1]]), localdir, file[2], bitrate, length])
-		
 		for directory in self.shares.keys():
 			if dir in directory and dir != directory:
 				files += self.DownloadDirectoryRecursive(directory, os.path.join(localdir, ""))
-	
-		return files	
+		return files
 		
 	def OnDownloadDirectoryTo(self, widget):
 		if self.selected_folder == None:
@@ -618,9 +610,7 @@ class UserBrowse(UserBrowseTab):
 
 		for directory in dir: # iterate over selected files
 			try:
- 				
 				self.DownloadDirectory(self.selected_folder, os.path.join(directory, ""))
-				
 			except IOError: # failed to open
 				self.frame.logMessage('failed to open %r for reading', directory) # notify user
 
@@ -630,9 +620,7 @@ class UserBrowse(UserBrowseTab):
 			return
 		for directory in dir: # iterate over selected files
 			try:
- 				
 				self.DownloadDirectory(self.selected_folder, os.path.join(directory, ""), 1)
-				
 			except IOError: # failed to open
 				self.frame.logMessage('failed to open %r for reading', directory) # notify user
 
@@ -641,7 +629,22 @@ class UserBrowse(UserBrowseTab):
 		if dir == None or dir not in self.shares:
 			return
 		ldir = prefix + dir.split("\\")[-1]
-		for file in self.shares[dir]:
+		priorityfiles = []
+		normalfiles = []
+		if self.frame.np.config.sections["transfers"]["prioritize"]:
+			for file in self.shares[dir]:
+				(junk, sep, ext) = file[1].rpartition('.')
+				if sep and ext in ['sfv','md5','nfo']:
+					priorityfiles.append(file)
+				else:
+					normalfiles.append(file)
+		else:
+			normalfiles = self.shares[dir][:]
+		if self.frame.np.config.sections["transfers"]["reverseorder"]:
+			deco = [(x[1], x) for x in normalfiles]
+			deco.sort(reverse=True)
+			normalfiles = [x for junk, x in deco]
+		for file in priorityfiles + normalfiles:
 			length = bitrate = None
 			attrs = file[4]
 			if attrs != []:
@@ -652,13 +655,12 @@ class UserBrowse(UserBrowseTab):
 					rl = int(attrs[1])
 				except:
 					rl = 0
-				length = "%i:%02i" % (int(rl / 60), rl % 60)
+				length = "%i:%02i" % (int(rl // 60), rl % 60)
 			self.frame.np.transfers.getFile(self.user, "\\".join([dir, file[1]]), ldir, size=file[2], bitrate=bitrate, length=length)
 		if not recurse:
 			return
 		for directory in self.shares.keys():
 			if dir in directory and dir != directory:
-	
 				self.DownloadDirectory(directory, os.path.join(ldir, ""), recurse)
 
 
@@ -681,21 +683,16 @@ class UserBrowse(UserBrowseTab):
 					rl = int(attrs[1])
 				except:
 					rl = 0
-				length = "%i:%02i" % (int(rl / 60), rl % 60)
+				length = "%i:%02i" % (int(rl // 60), rl % 60)
 			self.frame.np.transfers.getFile(self.user, path, prefix, size=size, bitrate=bitrate, length=length)
 
-	
 	def OnUploadDirectoryRecursiveTo(self, widget):
 		self.OnUploadDirectoryTo(widget, recurse=1)
-	
 	def UploadDirectoryTo(self, user, dir, recurse = 0):
 		if dir == "" or dir is None or user is None or user == "":
 			return
-
 		ldir = dir.split("\\")[-1]
-
 		if dir in self.shares.keys():
-
 			for file in self.shares[dir]:
 				path = "\\".join([dir, file[1]])
 				size = file[2]
@@ -703,7 +700,6 @@ class UserBrowse(UserBrowseTab):
 				self.frame.np.transfers.checkUploadQueue()
 		if not recurse:
 			return
-
 		for directory in self.shares.keys():
 			if dir in directory and dir != directory:
 				self.UploadDirectoryTo(user, directory, recurse)
@@ -743,38 +739,37 @@ class UserBrowse(UserBrowseTab):
 				self.frame.np.transfers.checkUploadQueue()
 			
 	def OnPlayFiles(self, widget, prefix = ""):
+		start_new_thread(self._OnPlayFiles, (widget, prefix))
+	def _OnPlayFiles(self, widget, prefix = ""):
 		path = self.selected_folder.replace("\\", os.sep)
 		executable = self.frame.np.config.sections["players"]["default"]
 		if "$" not in executable:
 			return
-		commandargs = executable.split(" ")
-		pos = commandargs.index("$")
 		for fn in self.selected_files:
 			file = os.sep.join([path, fn])
-			command = commandargs
-			command[pos] = file
 			if os.path.exists(file):
-				os.spawnlp(os.P_NOWAIT, command[0], *command)
+				executeCommand(executable, file, background=False)
 		
 	def OnDownloadFilesTo(self, widget):
 		
-		path, subdir = self.selected_folder.rsplit("\\", 1)
-		ldir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], create=True, name=subdir)
+		basedir, subdir = self.selected_folder.rsplit("\\", 1)
+		path = os.path.join(self.frame.np.config.sections["transfers"]["downloaddir"], subdir)
+		if os.path.exists(path) and os.path.isdir(path):
+			ldir = ChooseDir(self.frame.MainWindow, path)
+		else:
+			ldir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"])
 
 		if ldir is None:
 			return
-
 		for directory in ldir: # iterate over selected files
 			try:
- 				self.OnDownloadFiles(widget, directory)
-				
+				self.OnDownloadFiles(widget, directory)
 			except IOError: # failed to open
 				self.frame.logMessage('failed to open %r for reading', directory) # notify user
 
 	def FindMatches(self):
 		self.search_list = []
 		for directory, files in self.shares.items():
-		
 			if self.query in directory.lower():
 				if directory not in self.search_list:
 					self.search_list.append(directory)
@@ -792,7 +787,6 @@ class UserBrowse(UserBrowseTab):
 			if self.query == "":
 				return
 			self.FindMatches()
-		
 
 		dir = self.selected_folder
 		
@@ -802,18 +796,13 @@ class UserBrowse(UserBrowseTab):
 			self.search_list.sort()
 			directory = self.search_list[self.search_position]
 			path = self.DirStore.get_path( self.directories[directory] )
-			
-
 			self.FolderTreeView.expand_to_path(path)
-
 			self.FolderTreeView.set_cursor(path)
-
 			# Get matching files in the current directory
 			resultfiles = []
 			for file in self.files.keys():
 				if query in file.lower():
 					resultfiles.append(file)
-					
 			sel = self.FileTreeView.get_selection()
 			sel.unselect_all()
 			l = 1
@@ -826,14 +815,10 @@ class UserBrowse(UserBrowseTab):
 					# Position cursor at first match
 					self.FileTreeView.scroll_to_cell(path, None, True, 0.5, 0.5)
 					l = 0
-
-
 		else:
 			self.search_position = 0
-				
 
 	def OnClose(self, widget):
-		
 		self.userbrowses.remove_page(self.Main)
 		del self.userbrowses.users[self.user]
 		self.frame.np.ClosePeerConnection(self.conn)
@@ -865,8 +850,8 @@ class UserBrowse(UserBrowseTab):
 		path = self.selected_folder.replace("\\", os.sep)
 		executable = self.frame.np.config.sections["ui"]["filemanager"]
 		if "$" in executable:
-			os.system("%s &" % executable.replace("$", "\"%s\"" % path))
-			
+			executeCommand(executable, path)
+	
 	def OnEncodingChanged(self, widget):
 		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 6:
 			# PyGTK 2.6
