@@ -16,6 +16,10 @@
 # Original copyright below
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
+from __future__ import division
+
+import webbrowser
+
 import gtk
 import gobject
 import pango
@@ -30,7 +34,7 @@ from struct import unpack
 import imghdr
 
 from pynicotine import slskmessages
-from pynicotine.utils import _
+from pynicotine.utils import _, executeCommand
 
 DECIMALSEP = ""
 
@@ -194,8 +198,9 @@ def header_toggle(menuitem, column):
 
 	
 def ScrollBottom(widget):
-	widget.need_scroll = 1
 	va = widget.get_vadjustment()
+	if va is None:
+		return False
 	va.set_value(va.upper - va.page_size)
 	return False
 
@@ -205,12 +210,16 @@ def UrlEvent(tag, widget, event, iter, url):
 			url = "http://" + url
 		protocol = url[:url.find(":")]
 		if protocol in PROTOCOL_HANDLERS:
+			if NICOTINE.browser is not None and NICOTINE.np.config.sections["ui"]["open_in_mozembed"] and protocol == "http":
+				NICOTINE.browser.load_url(url, 0)
+				tag.last_event_type = event.type
+				return
 			if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
 				PROTOCOL_HANDLERS[protocol](url.strip())
+			elif PROTOCOL_HANDLERS[protocol]:
+				executeCommand(PROTOCOL_HANDLERS[protocol], url)
 			else:
-				cmd = '%s &' % (PROTOCOL_HANDLERS[protocol] % url)
-
-				os.system( cmd)
+				webbrowser.open(url)
 		else:
 			try:
 				import gnomevfs
@@ -230,6 +239,7 @@ def UrlEvent(tag, widget, event, iter, url):
 	tag.last_event_type = event.type
 
 def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, timestamp_format = "%H:%M:%S", username=None, usertag=None, scroll=True):
+	line = str(line) # Error messages are sometimes tuples
 	def _makeurltag(buffer, tag, url):
 		props = {}
 
@@ -251,10 +261,11 @@ def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, tim
 	scrolledwindow = textview.get_parent()
 	va = scrolledwindow.get_vadjustment()
 	bottom = va.value >= (va.upper - int(va.page_size*1.5))
-		
+
 	buffer = textview.get_buffer()
 	linenr = buffer.get_line_count()
 	ME = 0
+
 	if line.startswith("* "):
 		ME = 1
 	if NICOTINE.np.config.sections["logging"]["timestamps"] and showstamp:
@@ -312,10 +323,7 @@ def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, tim
 		else:
 			_append(buffer, line, tag)
 	
-	if not hasattr(scrolledwindow, "need_scroll"):
-		scrolledwindow.need_scroll = 1
-	if scroll and bottom and scrolledwindow.need_scroll:
-		scrolledwindow.need_scroll = 0
+	if scroll and bottom:
 		gobject.idle_add(ScrollBottom, scrolledwindow)
 	
 	return linenr
@@ -492,8 +500,9 @@ class ImageLabel(gtk.HBox):
 		return self.label.get_text()
 	
 class IconNotebook(gtk.Notebook):
-	def __init__(self, images, angle = 0, tabclosers = False, show_image = True):
+	def __init__(self, images, angle = 0, tabclosers = False, show_image = True, reorderable = True):
 		self.tabclosers = tabclosers
+		self.reorderable = reorderable
 		gtk.Notebook.__init__(self)
 		self.images = images
 		self._show_image = show_image
@@ -503,7 +512,16 @@ class IconNotebook(gtk.Notebook):
 		self.connect("key_press_event", self.OnKeyPress)
 		self.set_scrollable(True)
 		self.angle = angle
-		
+	
+	def set_reorderable(self, reorderable):
+		self.reorderable = reorderable
+		for data in self.pages:
+			page, label_tab, status, label_tab_menu = data
+			try:
+				self.set_tab_reorderable(page, self.reorderable)
+			except:
+				pass
+			
 	def set_tab_closers(self, closers):
 
 		self.tabclosers = closers
@@ -553,7 +571,7 @@ class IconNotebook(gtk.Notebook):
 		eventbox.connect('button_press_event', self.on_tab_click, page)
 		gtk.Notebook.append_page_menu(self, page, eventbox, label_tab_menu)
 		try:
-			self.set_tab_reorderable(page, True)
+			self.set_tab_reorderable(page, self.reorderable)
 			#self.set_tab_detachable(page, True)
 		except:
 			# Old PyGTK2
@@ -616,7 +634,7 @@ class IconNotebook(gtk.Notebook):
 		pass
 	
 	def attach_tab(self, page, destroying=False):
-		pagewidget = label_tab = label_tab_menu = label = None
+		pagewidget = label_tab = label_tab_menu = label = status = None
 
 		for item in self.detached_tabs:
 			if item[0] is page:
@@ -644,7 +662,7 @@ class IconNotebook(gtk.Notebook):
 		eventbox.connect('button_press_event', self.on_tab_click, page)
 		gtk.Notebook.append_page_menu(self, pagewidget, eventbox, label_tab_menu)
 		try:
-			self.set_tab_reorderable(page, True)
+			self.set_tab_reorderable(page, self.reorderable)
 		except:
 			# Old PyGTK2
 			pass
@@ -754,6 +772,7 @@ class PopupMenu(gtk.Menu):
 		self.user = None
 		self.useritem = None
 		self.handlers = {}
+		self.editing = False
 		
 	def setup(self, *items):
 		for item in items:
@@ -762,7 +781,10 @@ class PopupMenu(gtk.Menu):
 			elif item[0] == "USER":
 				menuitem = gtk.MenuItem(item[1])
 				self.useritem = menuitem
-				menuitem.set_sensitive(False)
+				if len(item) >= 3:
+					self.handlers[menuitem] = menuitem.connect("activate", item[2])
+				else:
+					menuitem.set_sensitive(False)
 			elif item[0] == 1:
 				menuitem = gtk.MenuItem(item[1])
 				menuitem.set_submenu(item[2])
@@ -779,7 +801,9 @@ class PopupMenu(gtk.Menu):
 			elif item[0] == 2:
 				menuitem = gtk.ImageMenuItem(item[1])
 				menuitem.set_submenu(item[2])
-				if item[3] is not None:
+				if len(item) == 5 and item[4] is not None and item[3] is not None:
+					self.handlers[menuitem] = menuitem.connect("activate", item[3], item[4])
+				elif item[3] is not None:
 					self.handlers[menuitem] = menuitem.connect("activate", item[3])
 				img = gtk.image_new_from_stock(item[4], gtk.ICON_SIZE_MENU)
 				menuitem.set_image(img)
@@ -797,7 +821,9 @@ class PopupMenu(gtk.Menu):
 					menuitem.set_image(img)
 				else:
 					menuitem = gtk.MenuItem(item[0])
-				if item[1] is not None:
+				if len(item) >= 4 and item[3] is not None and item[1] is not None:
+					self.handlers[menuitem] = menuitem.connect("activate", item[1], item[3])
+				elif item[1] is not None:
 					self.handlers[menuitem] = menuitem.connect("activate", item[1])
 			self.append(menuitem)
 			menuitem.show()
@@ -826,13 +852,15 @@ class PopupMenu(gtk.Menu):
 	def OnSearchUser(self, widget):
 		self.frame.SearchMethod.set_active_iter(self.frame.searchmethods[_("User")])
 		self.frame.UserSearchCombo.child.set_text(self.user)
-		self.frame.MainNotebook.set_current_page(4)
+		self.frame.ChangeMainPage(None, "search")
 		
 	def OnSendMessage(self, widget):
 		self.frame.privatechats.SendMessage(self.user, None, 1)
-		self.frame.MainNotebook.set_current_page(1)
+		self.frame.ChangeMainPage(None, "private")
 	
 	def OnShowIPaddress(self, widget):
+		if self.user not in self.frame.np.ip_requested:
+			self.frame.np.ip_requested.append(self.user)
 		self.frame.np.queue.put(slskmessages.GetPeerAddress(self.user))
 	
 	def OnGetUserInfo(self, widget):
@@ -840,20 +868,52 @@ class PopupMenu(gtk.Menu):
 	
 	def OnBrowseUser(self, widget):
 		self.frame.BrowseUser(self.user)
+		
+		
+	def OnPrivateRoomAddUser(self, widget, room):
+		self.frame.PrivateRoomAddUser(room, self.user)
+		
+	def OnPrivateRoomRemoveUser(self, widget, room):
+		self.frame.PrivateRoomRemoveUser(room, self.user)
 	
+	def OnPrivateRoomAddOperator(self, widget, room):
+		self.frame.PrivateRoomAddOperator(room, self.user)
+		
+	def OnPrivateRoomRemoveOperator(self, widget, room):
+		self.frame.PrivateRoomRemoveOperator(room, self.user)
+		
 	def OnAddToList(self, widget):
+		if self.editing: return
 		if widget.get_active():
 			self.frame.userlist.AddToList(self.user)
 		else:
 			self.frame.userlist.RemoveFromList(self.user)
 	
 	def OnBanUser(self, widget):
+		if self.editing: return
 		if widget.get_active():
 			self.frame.BanUser(self.user)
 		else:
 			self.frame.UnbanUser(self.user)
+			
+	def OnBlockUser(self, widget):
+		if self.editing: return
 	
+		if widget.get_active():
+			self.frame.OnBlockUser(self.user)
+		else:
+			self.frame.OnUnBlockUser(self.user)
+
+	def OnIgnoreIP(self, widget):
+		if self.editing: return
+	
+		if widget.get_active():
+			self.frame.OnIgnoreIP(self.user)
+		else:
+			self.frame.OnUnIgnoreIP(self.user)
+
 	def OnIgnoreUser(self, widget):
+		if self.editing: return
 		if widget.get_active():
 			self.frame.IgnoreUser(self.user)
 		else:
@@ -862,12 +922,15 @@ class PopupMenu(gtk.Menu):
 	def OnVersion(self, widget):
 		self.frame.privatechats.SendMessage(self.user, "\x01VERSION\x01")
 		
+	def OnCopyUser(self, widget):
+		self.frame.clip.set_text(self.user)
+		
 	def OnGivePrivileges(self, widget):
 		self.frame.np.queue.put(slskmessages.CheckPrivileges())
 		if self.frame.np.privileges_left is None:
 			days = _("Unknown")
 		else:
-			days = self.frame.np.privileges_left / 60 / 60 / 24
+			days = self.frame.np.privileges_left // 60 // 60 // 24
 			
 		text = InputDialog(self.frame.MainWindow, _("Give privileges")+" "+_("to %(user)s") %{"user": self.user}, _("Give how many days of global privileges to this user?") + " ("+ _("%(days)s days left") %{'days':days} +")" )
 		if text:
@@ -880,7 +943,7 @@ class PopupMenu(gtk.Menu):
 
 def InputDialog(parent, title, message, default = ""):
 	dlg = gtk.Dialog(title = title, parent = parent,
-		buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
+		buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
 	dlg.set_default_response(gtk.RESPONSE_OK)
 	
 	dlg.set_border_width(10)
@@ -1019,7 +1082,39 @@ def fixpath(path):
 		return path
 	except:
 		return path
+
+
+def HumanSize(number):
+	try:
+		s = float(int(number))
+		if s >= 1000*1024*1024:
+			r = _("%.2f GiB") % (s / (1024.0*1024.0*1024.0))
+		elif s >= 1000*1024:
+			r = _("%.2f MiB") % (s / (1024.0*1024.0))
+		elif s >= 1000:
+			r = _("%.2f KiB") % (s / 1024.0)
+		else:
+			r = _("%d Bytes") % s
+		return r
+	except Exception, e:
+		return number
 		
+def HumanSpeed(number):
+	try:
+		s = float(int(number))
+		if s >= 1000*1024*1024:
+			r = _("%.2f GiB/s") % (s / (1024.0*1024.0*1024.0))
+		elif s >= 1000*1024:
+			r = _("%.2f MiB/s") % (s / (1024.0*1024.0))
+		elif s >= 1000:
+			r = _("%.2f KiB/s") % (s / 1024.0)
+		else:
+			r = _("%d B/s") % (number)
+		return r
+	except Exception, e:
+		return number
+
+	
 def Humanize(number):
 	fashion = DECIMALSEP
 	if fashion == "" or fashion == "<None>":
@@ -1256,7 +1351,7 @@ class ImportWinSlskConfig:
 		return 1
   
 	def check_slskdir(self):
-		
+		path = ""
 		# we check if the file queue2.cfg exists under the slsk dir
 		if self.Queue:
 			path = self.winpath('queue2.cfg')
