@@ -22,8 +22,9 @@
 import struct
 import types
 import zlib
-from utils import _
 import os
+from utils import _
+from logfacility import log
 
 """ This module contains message classes, that networking and UI thread
 exchange. Basically there are three types of messages: internal messages,
@@ -106,7 +107,7 @@ class UploadFile(InternalMessage):
 		self.offset = offset
 
 class FileError(InternalMessage):
-	""" Sent by networking thread to indicate that a file error occured during
+	""" Sent by networking thread to indicate that a file error occurred during
 	filetransfer. """
 	def __init__(self, conn = None, file = None, strerror = None):
 		self.conn = conn
@@ -148,7 +149,21 @@ class Notify(InternalMessage):
 class InternalData(InternalMessage):
 	def __init__(self, msg):
 		self.msg = msg
-		
+
+class DebugMessage(InternalMessage):
+	def __init__(self, msg, debugLevel = None):
+		''' debugLevel Options
+		0/None - Normal messages and (Human-Readable) Errors
+		1 - Warnings & Tracebacks
+		2 - Search Results
+		3 - Peer Connections
+		4 - Message Contents
+		5 - Transfers
+		6 - Connection, Bandwidth and Usage Statistics
+		'''
+		self.msg = msg
+		self.debugLevel = debugLevel
+
 class SlskMessage:
 	""" This is a parent class for all protocol messages. """
 	
@@ -173,7 +188,7 @@ class SlskMessage:
 			else:
 				return start, None
 		except struct.error, error:
-			print self.__class__, error
+			log.addwarning("%s %s" % (self.__class__, error))
 			return start, None
 
 	def packObject(self, object):
@@ -186,21 +201,21 @@ class SlskMessage:
 		elif type(object) is types.StringType:
 			return struct.pack("<i", len(object))+object
 		elif type(object) is types.UnicodeType:
-			print _("Warning: networking thread has to convert unicode string %(object)s message %(type)s") % {'object':object, 'type':self.__class__}
+			log.addwarning(_("Warning: networking thread has to convert unicode string %(object)s message %(type)s") % {'object':object, 'type':self.__class__})
 			encoded = object.encode("utf-8",'replace')
 			return struct.pack("<i", len(encoded))+encoded
-		print _("Warning: unknown object type %s") % type(object) +" "+ ("in message %(type)s") % {'type':self.__class__}
+		log.addwarning(_("Warning: unknown object type %s") % type(object) +" "+ ("in message %(type)s") % {'type':self.__class__})
 		return ""
         
 	def makeNetworkMessage(self):
 		""" Returns binary array, that can be sent over the network"""
-		print _("Empty message made, class %s") % self.__class__
+		log.addwarning(_("Empty message made, class %s") % (self.__class__,))
 		return None
     
 	def parseNetworkMessage(self, message):
 		""" Extracts information from the message and sets up fields 
 		in an object"""
-		print _("Can't parse incoming messages, class %s") % self.__class__
+		log.addwarning(_("Can't parse incoming messages, class %s") % (self.__class__,))
 	
 	def strrev(self, str):
 		strlist = list(str)
@@ -212,6 +227,11 @@ class SlskMessage:
 		strlist.reverse()
 		return '.'.join(strlist)
 	
+	def debug(self, message=None):
+		print self, self.__dict__	
+		if message:
+			print "Message contents:", message.__repr__()
+		
 class ServerMessage(SlskMessage):
 	pass
 
@@ -232,11 +252,16 @@ class Login(ServerMessage):
 		self.ip = None
 		
 	def makeNetworkMessage(self):
-		import md5
-		m = md5.new()
+		try:
+			import hashlib
+			m = hashlib.md5()
+		except:
+			import md5
+			m = md5.new()
 		m.update(self.username+self.passwd)
 		md5hash = m.hexdigest()
-		message = self.packObject(self.username)+ self.packObject(self.passwd) + self.packObject(self.version) + self.packObject(md5hash) + self.packObject(1)
+		message = self.packObject(self.username)+ self.packObject(self.passwd) + self.packObject(self.version) + self.packObject(md5hash) + self.packObject(17)
+		#print message.__repr__()
 		return message
 
 
@@ -253,7 +278,7 @@ class Login(ServerMessage):
 				pos, self.ip = pos+4, socket.inet_ntoa(self.strrev(message[pos:pos+4]))
 				# Unknown number
 			except Exception, error:
-				print "Error unpacking IP address", error
+				log.addwarning("Error unpacking IP address: %s" % (error,))
 			try:
 				# MD5 hexdigest of the password you sent
 				if len(message[pos:]) > 0:
@@ -263,6 +288,18 @@ class Login(ServerMessage):
 				# Not an official client on the official server
 				pass
 
+class ChangePassword(ServerMessage):
+	""" We sent this to the server to change our password 
+	We receive a response if our password changes. """
+	
+	def __init__(self, password=None):
+		self.password = password
+
+	def makeNetworkMessage(self):
+		return self.packObject(self.password)
+
+	def parseNetworkMessage(self, message):
+		pos, self.password = self.getObject(message, types.StringType)
 
 class SetWaitPort(ServerMessage):
 	""" Send this to server to indicate port number that we listen on."""
@@ -316,6 +353,14 @@ class AddUser(ServerMessage):
 			if len(message[pos:]) > 0:
 				pos, self.country = self.getObject(message, types.StringType, pos)
 
+class Unknown6(ServerMessage):
+	def __init__(self):
+		pass
+
+	def parseNetworkMessage(self, message):
+		self.debug()
+		pass
+
 class RemoveUser(ServerMessage):
 	""" Used when we no longer want to be kept updated about a user's status."""
 	def __init__(self, user = None):
@@ -367,10 +412,27 @@ class AckNotifyPrivileges(ServerMessage):
 	def __init__(self, token = None):
 		self.token = token
 	def parseNetworkMessage(self, message):
-		pos, self.token = self.getObject(message, types.IntType)	
+		pos, self.token = self.getObject(message, types.IntType)
 	def makeNetworkMessage(self):
 		return self.packObject(self.token)
-
+class JoinPublicRoom(ServerMessage):
+	"""We want to join the Public Chat"""
+	def __init__(self, unknown = 0):
+		self.unknown = unknown
+	def makeNetworkMessage(self):
+		return self.packObject(self.unknown)
+class LeavePublicRoom(ServerMessage):
+	"""We want to leave the Public Chat"""
+	def __init__(self, unknown = 0):
+		self.unknown = unknown
+	def makeNetworkMessage(self):
+		return self.packObject(self.unknown)
+class PublicRoomMessage(ServerMessage):
+	"""The server sends us messages from random chatrooms"""
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.user = self.getObject(message, types.StringType, pos)
+		pos, self.msg = self.getObject(message, types.StringType, pos)
 class SayChatroom(ServerMessage):
 	""" Either we want to say something in the chatroom, or someone did."""
 	def __init__(self, room = None, msg = None):
@@ -401,10 +463,13 @@ class UserData:
 class JoinRoom(ServerMessage):
 	""" Server sends us this message when we join a room. Contains users list
 	with data on everyone."""
-	def __init__(self, room = None):
+	def __init__(self, room = None, private = None):
 		self.room = room
+		self.private = private
 	
 	def makeNetworkMessage(self):
+		if self.private is not None:
+			return self.packObject(self.room) + self.packObject(self.private)
 		return self.packObject(self.room)
 	
 	def parseNetworkMessage(self, message):
@@ -442,6 +507,191 @@ class JoinRoom(ServerMessage):
 			usersdict[i[0]] = UserData(i[1:])
 		return usersdict
 
+
+class PrivateRoomUsers(ServerMessage):
+	""" We get this when we've created a private room."""
+	def __init__(self, room = None, numusers = None, users = None):
+		self.room = room
+		self.numusers = numusers
+		self.users = users
+			
+	#def makeNetworkMessage(self):
+		#return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.numusers = self.getObject(message, types.IntType, pos)
+		self.users = []
+		for i in range(self.numusers):
+			pos, user = self.getObject(message, types.StringType, pos)
+			self.users.append(user)
+
+	
+class PrivateRoomOwned(ServerMessage):
+	""" We get this when we've created a private room."""
+	def __init__(self, room = None, number = None):
+		self.room = room
+		self.number = number
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.number = self.getObject(message, types.IntType, pos)
+		self.operators = []
+		for i in range(self.number):
+			pos, user = self.getObject(message, types.StringType, pos)
+			self.operators.append(user)
+		#self.debug()
+	
+		
+class PrivateRoomAddUser(ServerMessage):
+	""" We get / receive this when we add a user to a private room."""
+	def __init__(self, room = None, user = None):
+		self.room = room
+		self.user = user
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room) + self.packObject(self.user)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.user = self.getObject(message, types.StringType, pos)
+	
+class PrivateRoomDismember(ServerMessage):
+	""" We do this to remove our own membership of a private room."""
+	def __init__(self, room = None):
+		self.room = room
+
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		
+class PrivateRoomDisown(ServerMessage):
+	""" We do this to stop owning a private room."""
+	def __init__(self, room = None):
+		self.room = room
+
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+
+		
+class PrivateRoomSomething(ServerMessage):
+	""" We get this when we've removed a user from a private room."""
+	def __init__(self, room = None):
+		self.room = room
+
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		#pos, self.user = self.getObject(message, types.StringType, pos)
+		#print message[pos:].__repr__()
+		
+class PrivateRoomRemoveUser(ServerMessage):
+	""" We get this when we've removed a user from a private room."""
+	def __init__(self, room = None, user = None):
+		self.room = room
+		self.user = user
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room) + self.packObject(self.user)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.user = self.getObject(message, types.StringType, pos)
+		
+class PrivateRoomAdded(ServerMessage):
+	""" We are sent this when we are added to a private room."""
+	def __init__(self, room = None):
+		self.room = room
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		self.room = self.getObject(message, types.StringType)[1]
+		
+class PrivateRoomRemoved(ServerMessage):
+	""" We are sent this when we are removed from a private room."""
+	def __init__(self, room = None):
+		self.room = room
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		self.room = self.getObject(message, types.StringType)[1]
+		
+class PrivateRoomToggle(ServerMessage):
+	""" We send this when we want to enable or disable invitations to private rooms"""
+	def __init__(self, enabled = None):
+		self.enabled = enabled
+	
+	def makeNetworkMessage(self):
+		return chr(self.enabled)
+	
+	def parseNetworkMessage(self, message):
+		# When this is received, we store it in the config, and disable the appropriate menu item
+		pos, self.enabled = 1, bool(ord(message[0]))
+
+
+
+
+class PrivateRoomAddOperator(ServerMessage):
+	""" We send this to add private room operator abilities to a user"""
+	def __init__(self, room = None, user = None):
+		self.room = room
+		self.user = user
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room) + self.packObject(self.user)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.user = self.getObject(message, types.StringType, pos)
+		
+class PrivateRoomRemoveOperator(ServerMessage):
+	""" We send this to remove privateroom operator abilities from a user"""
+	def __init__(self, room = None, user = None):
+		self.room = room
+		self.user = user
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room) + self.packObject(self.user)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		pos, self.user = self.getObject(message, types.StringType, pos)
+		
+class PrivateRoomOperatorAdded(ServerMessage):
+	""" We receive this when given privateroom operator abilities"""
+	def __init__(self, room = None):
+		self.room = room
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		#pos, self.username = self.getObject(message, types.StringType, pos)
+		
+class PrivateRoomOperatorRemoved(ServerMessage):
+	""" We receive this when privateroom operator abilities are removed"""
+	def __init__(self, room = None):
+		self.room = room
+	
+	def makeNetworkMessage(self):
+		return self.packObject(self.room)
+	
+	def parseNetworkMessage(self, message):
+		pos, self.room = self.getObject(message, types.StringType)
+		#pos, self.username = self.getObject(message, types.StringType, pos)
+		
 class LeaveRoom(ServerMessage):
 	""" We send this when we want to leave a room."""
 	def __init__(self, room = None):
@@ -575,6 +825,8 @@ class FileSearch(ServerMessage):
 	def __init__(self, requestid = None, text = None):
 		self.searchid = requestid
 		self.searchterm = text
+		if text:
+			self.searchterm = ' '.join([x for x in text.split() if x != '-'])
 	
 	def makeNetworkMessage(self):
 		return self.packObject(self.searchid)+self.packObject(self.searchterm)
@@ -606,7 +858,7 @@ class SendSpeed(ServerMessage):
 
 class SendUploadSpeed(ServerMessage):
 	""" We now send this after a finished upload to let the server update
-	the spped statistics for a user"""
+	the speed statistics for a user"""
 	def __init__(self, speed = None):
 		self.speed = speed
 	
@@ -665,6 +917,7 @@ class PlaceInLineResponse(ServerMessage):
 		pos, self.user = self.getObject(message, types.StringType)
 		pos, self.req = self.getObject(message, types.IntType, pos)
 		pos, self.place = self.getObject(message, types.IntType, pos)
+		#print self.user, self.req, self.place
 
 class RoomAdded(ServerMessage):
 	""" Server tells us a new room has been added"""
@@ -696,7 +949,22 @@ class RoomList(ServerMessage):
 		for i in range(numusercounts):
 			pos, usercount = self.getObject(message, types.IntType, pos)
 			self.rooms[i][1] = usercount
-
+		self.privaterooms = []
+		
+		try:
+			if len(message[pos:]) == 0:
+				return
+			pos, numprivateroom = self.getObject(message, types.IntType, pos)
+			for i in range(numprivateroom):
+				pos, room = self.getObject(message, types.StringType, pos)
+				self.privaterooms.append([room, None])
+			pos, numprivateroomusers = self.getObject(message, types.IntType, pos)
+			for i in range(numprivateroomusers):
+				pos, usercount = self.getObject(message, types.IntType, pos)
+				self.privaterooms[i][1] = usercount
+		except Exception, error:
+			log.addwarning(_("Exception during parsing %(area)s: %(exception)s") % {'area':'RoomList', 'exception':error})
+			pass
 
 class ExactFileSearch(ServerMessage):
 	""" Someone is searching for a file with an exact name """
@@ -841,7 +1109,7 @@ class WishlistInterval(ServerMessage):
 		pos, self.seconds = self.getObject(message, types.IntType)
 
 class PrivilegedUsers(ServerMessage):
-	""" A list of thise who made a donation """
+	""" A list of those who made a donation """
 	def __init__(self):
 		pass
 	
@@ -934,7 +1202,7 @@ class UserInterests(ServerMessage):
 		return self.packObject(self.user)
 	
 	def parseNetworkMessage(self, message, pos=0):
-		# Recieve a users' interests
+		# Receive a users' interests
 		pos, self.user = self.getObject(message, types.StringType, pos)
 		pos, likesnum = self.getObject(message, types.IntType, pos)
 		self.likes = []
@@ -1006,7 +1274,7 @@ class SimilarUsers(ServerMessage):
 			pos, user = self.getObject(message, types.StringType, pos)
 			pos, rating = self.getObject(message, types.IntType, pos)
 			self.users[user] = rating
-
+		
 class ItemSimilarUsers(ServerMessage):
 	def __init__(self, thing = None):
 		self.thing = thing
@@ -1028,6 +1296,8 @@ class RoomSearch(ServerMessage):
 		self.room = room
 		self.searchid = requestid
 		self.searchterm = text
+		if text:
+			self.searchterm = ' '.join([x for x in text.split() if x != '-'])
 	
 	def makeNetworkMessage(self):
 		return self.packObject(self.room)+self.packObject(self.searchid)+self.packObject(self.searchterm)
@@ -1150,7 +1420,7 @@ class SharedFileList(PeerMessage):
 	def __init__(self, conn, shares = None):
 		self.conn = conn
 		self.list = shares
-	 	self.built = None
+		self.built = None
 		
 	def parseNetworkMessage(self, message, nozlib = 0):
 
@@ -1158,7 +1428,7 @@ class SharedFileList(PeerMessage):
 			try:
 				message=zlib.decompress(message)
 			except Exception, error:
-				print error
+				log.addwarning(_("Exception during parsing %(area)s: %(exception)s") % {'area':'SharedFileList', 'exception':error})
 				self.list={}
 				return
 
@@ -1189,7 +1459,7 @@ class SharedFileList(PeerMessage):
 	def makeNetworkMessage(self, nozlib = 0, rebuild=False):
 		# Elaborate hack, to save CPU
 		# Store packed message contents in self.built, and use
-		# instead of repacking it, unles rebuild is True
+		# instead of repacking it, unless rebuild is True
 		if not rebuild and self.built is not None:
 			return self.built
 		msg = ""
@@ -1249,7 +1519,7 @@ class FileSearchResult(PeerMessage):
 		try:
 			message = zlib.decompress(message)
 		except Exception, error:
-			print error
+			log.addwarning(_("Exception during parsing %(area)s: %(exception)s") % {'area':'FileSearchResult', 'exception':error})
 			self.list = {}
 			return
 	
@@ -1339,7 +1609,7 @@ class FolderContentsResponse(PeerMessage):
 		try:
 			message = zlib.decompress(message)
 		except Exception, error:
-			print error
+			log.addwarning(_("Exception during parsing %(area)s: %(exception)s") % {'area':'FolderContentsResponse', 'exception':error})
 			self.list = {}
 			return
 	#        f = open("ttt","w")
@@ -1521,14 +1791,78 @@ class DistribSearch(DistribMessage):
 		pos, self.searchid = self.getObject(message, types.IntType, pos)
 		pos, self.searchterm = self.getObject(message, types.StringType, pos)
 
-class DistribUnknown(DistribMessage):
+class DistribBranchLevel(DistribMessage):
 	def __init__(self, conn):
 		self.conn = conn
 	
 	def parseNetworkMessage(self, message):
-		pos, self.something = self.getObject(message, types.IntType)
+		pos, self.value = self.getObject(message, types.IntType)
+		#print message.__repr__()
+
+class DistribBranchRoot(DistribMessage):
+	def __init__(self, conn):
+		self.conn = conn
+	
+	def parseNetworkMessage(self, message):
+		#pos, self.value = self.getObject(message, types.IntType)
+		pos, self.user = self.getObject(message, types.StringType)
+		#print self.something, self.user
+
+class DistribChildDepth(DistribMessage):
+	def __init__(self, conn):
+		self.conn = conn
+	
+	def parseNetworkMessage(self, message):
+		pos, self.value = self.getObject(message, types.IntType)
+		#print self.something, self.user
+
+class DistribMessage9(DistribMessage):
+	def __init__(self, conn):
+		self.conn = conn
+	
+	def parseNetworkMessage(self, message):
+		#pos, self.value = self.getObject(message, types.IntType)
+		try:
+			x = zlib.decompress(message)
+		except:
+			self.debug()
+		#message =  x[4:]
+		#pos, self.user = self.getObject(message, types.StringType)
+		#self.debug()
+		#print self.something, self.user
+
+class BranchLevel(ServerMessage):
+	def __init__(self):
 		pass
 	
+	def parseNetworkMessage(self, message):
+		pos, self.value = self.getObject(message, types.IntType)
+		#print message.__repr__()
+
+class BranchRoot(ServerMessage):
+	def __init__(self):
+		pass
+	
+	def parseNetworkMessage(self, message):
+		#pos, self.value = self.getObject(message, types.IntType)
+		pos, self.user = self.getObject(message, types.StringType)
+		#print self.something, self.user
+
+class AcceptChildren(ServerMessage):
+	def __init__(self, enabled = None):
+		self.enabled = enabled
+
+	def makeNetworkMessage(self):
+		return chr(self.enabled)
+
+class ChildDepth(ServerMessage):
+	def __init__(self):
+		pass
+	
+	def parseNetworkMessage(self, message):
+		pos, self.value = self.getObject(message, types.IntType)
+
+
 class NetInfo(ServerMessage):
 	""" Information about what nodes have been added/removed in the network """
 	def __init__(self):
@@ -1555,7 +1889,22 @@ class SearchRequest(ServerMessage):
 		pos, self.user = self.getObject(message, types.StringType, pos)
 		pos, self.searchid = self.getObject(message, types.IntType, pos)
 		pos, self.searchterm = self.getObject(message, types.StringType, pos)
-	
+
+class UserPrivileged(ServerMessage):
+	""" Discover whether a user is privileged or not """
+	def __init__(self, user = None):
+		self.user = user
+		self.privileged = None
+
+	def makeNetworkMessage(self):
+		return self.packObject(self.user)
+
+
+	def parseNetworkMessage(self, message):
+		pos, self.user = self.getObject(message, types.StringType, pos)
+		pos, self.privileged = pos+1, bool(ord(message[pos]))
+
+
 class GivePrivileges(ServerMessage):
 	""" Give (part) of your privileges to another user on the network """
 	def __init__(self, user = None, days = None):
@@ -1564,3 +1913,11 @@ class GivePrivileges(ServerMessage):
 
 	def makeNetworkMessage(self):
 		return self.packObject(self.user) + self.packObject(self.days)
+
+class PopupMessage(object):
+	"""For messages that should be shown to the user prominently, for example
+	through a popup. Should be used sparsely."""
+	def __init__(self, title, message):
+		self.title = title
+		self.message = message
+
