@@ -23,6 +23,10 @@
 This module implements SoulSeek networking protocol.
 """
 
+from __future__ import division
+
+from math import floor
+
 from slskmessages import *
 import SocketServer
 import socket
@@ -34,7 +38,40 @@ import threading
 import struct
 
 from errno import EINTR
-from utils import _
+from utils import _, win32
+from logfacility import log
+
+MAXFILELIMIT = -1
+if win32:
+	import ctypes
+	MAXFILELIMIT = ctypes.cdll.msvcrt._getmaxstdio()
+try:
+	import resource
+	try:
+		(soft, MAXFILELIMIT) = resource.getrlimit(resource.RLIMIT_NOFILE)
+	except AttributeError:
+		pass
+except ImportError:
+	pass
+
+# OSX reports INFINITE as hard limit, but supports up to 10240
+# Solaris supposedly reports 65535 and actually supports this
+# Linux usually reports 1024 and supports this.
+if MAXFILELIMIT > 65535:
+	MAXFILELIMIT = 1024
+if MAXFILELIMIT > 0:
+	# Bumping soft limit
+	try:
+		resource.setrlimit(resource.RLIMIT_NOFILE, (MAXFILELIMIT, MAXFILELIMIT))
+	except:
+		pass
+	# Since most people have a limit of 1024 or higher we can
+	# set it to 90% of the max limit and still get a workable amount of
+	# connections. We cannot set it to 100% because our connection count
+	# doesn't agree with with the OS connection count (at least on Linux),
+	# maybe because closed connections aren't closed on the spot.
+	#
+	MAXFILELIMIT = max(int(floor(MAXFILELIMIT*0.9)), 100)
 
 class Connection:
 	"""
@@ -100,6 +137,7 @@ class SlskProtoThread(threading.Thread):
 		SetWaitPort:2,
 		GetPeerAddress:3,
 		AddUser:5,
+		Unknown6:6,
 		GetUserStatus:7,
 		SayChatroom:13,
 		JoinRoom:14,
@@ -123,14 +161,14 @@ class SlskProtoThread(threading.Thread):
 		Recommendations:54,
 		GlobalRecommendations:56,
 		UserInterests:57,
-		PlaceInLineResponse:60,
+		PlaceInLineResponse:60, # Depreciated?
 		RoomAdded:62,
 		RoomRemoved:63,
 		RoomList:64,
 		ExactFileSearch:65,
 		AdminMessage:66,
-		GlobalUserList:67,
-		TunneledMessage:68,
+		GlobalUserList:67, # Depreciated?
+		TunneledMessage:68,  # Depreciated?
 		PrivilegedUsers:69,
 		HaveNoParent:71,
 		SearchParent:73,
@@ -144,8 +182,8 @@ class SlskProtoThread(threading.Thread):
 		DistribAliveInterval:90,
 		AddToPrivileged:91,
 		CheckPrivileges:92,
-		CantConnectToPeer:1001,
 		SearchRequest:93,
+		AcceptChildren:100,
 		NetInfo:102,
 		WishlistSearch:103,
 		WishlistInterval:104,
@@ -160,9 +198,33 @@ class SlskProtoThread(threading.Thread):
 		RemoveThingIHate:118,
 		RoomSearch:120, 
 		SendUploadSpeed:121,
+		UserPrivileged:122,
 		GivePrivileges:123,
 		NotifyPrivileges:124,
 		AckNotifyPrivileges:125,
+		BranchLevel:126,
+		BranchRoot:127,
+		ChildDepth:129,
+		#AnotherStatus:10,
+		PrivateRoomUsers:133,
+		PrivateRoomAddUser:134,
+		PrivateRoomRemoveUser:135,
+		PrivateRoomDismember:136,
+		PrivateRoomDisown:137,
+		PrivateRoomSomething:138,
+		PrivateRoomAdded:139,
+		PrivateRoomRemoved:140,
+		PrivateRoomToggle:141,
+		ChangePassword:142,
+		PrivateRoomAddOperator:143,
+		PrivateRoomRemoveOperator:144,
+		PrivateRoomOperatorAdded:145,
+		PrivateRoomOperatorRemoved:146,
+		PrivateRoomOwned:148,
+		JoinPublicRoom:150,
+		LeavePublicRoom:151,
+		PublicRoomMessage:152,
+		CantConnectToPeer:1001,
 		}
 		
 	peercodes = {
@@ -187,7 +249,7 @@ class SlskProtoThread(threading.Thread):
 		Msg12547:12547
 		}
 
-	distribclasses = {0:DistribAlive, 3:DistribSearch, 4:DistribUnknown}
+	distribclasses = {0:DistribAlive, 3:DistribSearch, 4:DistribBranchLevel, 5: DistribBranchRoot, 7: DistribChildDepth, 9:DistribMessage9}
 
 	
 	def __init__(self, ui_callback, queue, config, eventprocessor):
@@ -220,7 +282,7 @@ class SlskProtoThread(threading.Thread):
 		# GeoIP Module
 		self.geoip = self._eventprocessor.geoip
 		listenport = None
-
+		self.lastsocketwarning = 0
 
 		for listenport in range(portrange[0], portrange[1]+1):
 			try:
@@ -233,6 +295,12 @@ class SlskProtoThread(threading.Thread):
 				break
 		if listenport is not None:
 			self.start()
+		else:
+			short = _("Could not bind to a local port, aborting connection")
+			long = _("The range you specified for client connection ports was %(low)s-%(high)s, but none of these were usable. Increase and/or move the range and restart Nicotine+.") % {'low':portrange[0], 'high':portrange[1]}
+			if  portrange[0] < 1024:
+				long += "\n\n" + _("Note that part of your range lies below 1024, this is usually not allowed on most operating systems with the exception of Windows.")
+			self._ui_callback([PopupMessage(short, long)])
 	
 	def _isUpload(self, conn):
 		return conn.__class__ is PeerConnection and conn.fileupl is not None
@@ -306,6 +374,7 @@ class SlskProtoThread(threading.Thread):
 				if p is not None:
 					numsockets += 1
 				numsockets += len(conns) + len(connsinprogress)
+
 				self._ui_callback([InternalData(numsockets)])
 				#print "Sockets open:", len(conns.keys()+connsinprogress.keys()+[p]+outsock), len(conns.keys()),  len(connsinprogress.keys())
 			except select.error, error:
@@ -313,12 +382,12 @@ class SlskProtoThread(threading.Thread):
 					# Error recieved; but we don't care :)
 					continue
 				# Error recieved; terminate networking loop
-				print error
+				print time.strftime("%H:%M:%S"), "select.error", error
 				self._want_abort = 1
-				self._ui_callback([_("Major Socket Error: Networking terminated! %s" % str(error)) ])
+				self._ui_callback([DebugMessage(_("Major Socket Error: Networking terminated! %s" % str(error))) ])
 			except ValueError, error:
 				# Possibly opened too many sockets
-				print error
+				print time.strftime("%H:%M:%S"), "select ValueError:",  error
 				continue
 				#print len(conns.keys()), len(connsinprogress.keys())
 			# Write Output
@@ -337,74 +406,73 @@ class SlskProtoThread(threading.Thread):
 				else:
 					ip, port = self.getIpPort(incaddr)
 					if self.ipBlocked(ip):
-						self._ui_callback(["ignore connection request from blocked ip address %s %s" %( ip, port)])
-						pass
+						self._ui_callback([DebugMessage("Ignoring connection request from blocked IP Address %s:%s" %( ip, port), 3)])
+						
 					else:
 						conns[incconn] = PeerConnection(incconn, incaddr, "", "")
 						self._ui_callback([IncConn(incconn, incaddr)])
 					
 			# Manage Connections
 			curtime = time.time()
-			for i in connsinprogress.keys()[:]:
+			for connection_in_progress in connsinprogress.keys()[:]:
 				try:
-					msgObj = connsinprogress[i].msgObj
-					if i in input:
-						i.recv(0)
+					msgObj = connsinprogress[connection_in_progress].msgObj
+					if connection_in_progress in input:
+						connection_in_progress.recv(0)
 				except socket.error, err:
 					self._ui_callback([ConnectError(msgObj, err)])
-					i.close()
-					del connsinprogress[i]
+					connection_in_progress.close()
+					del connsinprogress[connection_in_progress]
 				else:
-					if i in output:
-						if i is server_socket:
+					if connection_in_progress in output:
+						if connection_in_progress is server_socket:
 							conns[server_socket] = ServerConnection(server_socket, msgObj.addr, "","")
 							self._ui_callback([ServerConn(server_socket, msgObj.addr)])
 						else:
 							ip, port = self.getIpPort(msgObj.addr)
 							if self.ipBlocked(ip):
 								message = "Blocking peer connection in progress to IP: %(ip)s Port: %(port)s" % { "ip":ip, "port":port}
-								self._ui_callback([message])
-								i.close()
+								self._ui_callback([DebugMessage(message, 3)])
+								connection_in_progress.close()
 				
 							else:
-								conns[i] = PeerConnection(i, msgObj.addr, "", "", msgObj.init)
-								self._ui_callback([OutConn(i, msgObj.addr)])
+								conns[connection_in_progress] = PeerConnection(connection_in_progress, msgObj.addr, "", "", msgObj.init)
+								self._ui_callback([OutConn(connection_in_progress, msgObj.addr)])
 							
 						
-						del connsinprogress[i]
-					
+						del connsinprogress[connection_in_progress]
 			# Process Data
-			for i in conns.keys()[:]:
-				ip, port = self.getIpPort(conns[i].addr)
-				if self.ipBlocked(ip) and i is not self._server_socket:
+			for connection in conns.keys()[:]:
+				ip, port = self.getIpPort(conns[connection].addr)
+				if self.ipBlocked(ip) and connection is not self._server_socket:
 					message = "Blocking peer connection to IP: %(ip)s Port: %(port)s" % { "ip":ip, "port":port}
 					print message
-					self._ui_callback([message])
-					i.close()
-					del conns[i]
+					self._ui_callback([DebugMessage(message, 3)])
+					connection.close()
+					del conns[connection]
 					continue
 
-				if i in input:
+				if connection in input:
 					try:
-						self.readData(conns, i)
+						self.readData(conns, connection)
 					except socket.error, err:
-						self._ui_callback([ConnectError(conns[i], err)])
-				if i in conns and len(conns[i].ibuf) > 0:
-					if i is server_socket:
+						self._ui_callback([ConnectError(conns[connection], err)])
+				if connection in conns and len(conns[connection].ibuf) > 0:
+					if connection is server_socket:
 						msgs, conns[server_socket].ibuf = self.process_server_input(conns[server_socket].ibuf)
 						self._ui_callback(msgs)
 					else:
-						if conns[i].init is None or conns[i].init.type not in ['F', 'D']:
-							msgs, conns[i] = self.process_peer_input(conns[i], conns[i].ibuf)
+						if conns[connection].init is None or conns[connection].init.type not in ['F', 'D']:
+							msgs, conns[connection] = self.process_peer_input(conns[connection], conns[connection].ibuf)
 							self._ui_callback(msgs)
-						if conns[i].init is not None and conns[i].init.type == 'F':
-							msgs, conns[i] = self.process_file_input(conns[i], conns[i].ibuf)
+						if conns[connection].init is not None and conns[connection].init.type == 'F':
+							msgs, conns[connection] = self.process_file_input(conns[connection], conns[connection].ibuf)
 							self._ui_callback(msgs)
-						if conns[i].init is not None and conns[i].init.type == 'D':
-							msgs, conns[i] = self.process_distrib_input(conns[i], conns[i].ibuf)
+						if conns[connection].init is not None and conns[connection].init.type == 'D':
+							msgs, conns[connection] = self.process_distrib_input(conns[connection], conns[connection].ibuf)
 							self._ui_callback(msgs)
-						if conns[i].conn == None:
-							del conns[i]
+						if conns[connection].conn == None:
+							del conns[connection]
 
 			# ---------------------------
 			# Server Pings used to get us banned
@@ -413,20 +481,23 @@ class SlskProtoThread(threading.Thread):
 			curtime = time.time()
 			connsockets = len(conns.keys())
 			num = 0
-			for i in conns.keys()[:]:
-				if i is not server_socket and i is not p:
-					if curtime - conns[i].lastactive > 60:
+			for connection in conns.keys()[:]:
+				if connection is not server_socket and connection is not p:
+					if curtime - conns[connection].lastactive > 60:
 
-						self._ui_callback([ConnClose(i, conns[i].addr)])
-						i.close()
+						self._ui_callback([ConnClose(connection, conns[connection].addr)])
+						connection.close()
 						#print "Closed_run", conns[i].addr
-						del conns[i]
-				else:
+						del conns[connection]
+			
 					#  Was 30 seconds
-					if curtime - conns[server_socket].lastping > 120:
-						conns[server_socket].lastping = curtime
-						queue.put(ServerPing())
+			
 				num += 1
+			if server_socket in conns.keys():
+				if curtime - conns[server_socket].lastping > 120:
+					conns[server_socket].lastping = curtime
+					queue.put(ServerPing())
+					
 			self._ui_callback([])
 
 		# Close Server Port
@@ -517,7 +588,7 @@ class SlskProtoThread(threading.Thread):
 		conns[i].lastactive = time.time()
 		data = i.recv(conns[i].lastreadlength)
 		conns[i].ibuf = conns[i].ibuf + data
-		if len(data) >= conns[i].lastreadlength/2:
+		if len(data) >= conns[i].lastreadlength//2:
 			conns[i].lastreadlength = conns[i].lastreadlength * 2 
 		if not data:
 			self._ui_callback([ConnClose(i, conns[i].addr)])
@@ -736,13 +807,17 @@ class SlskProtoThread(threading.Thread):
 				i.starttime = curtime
 				i.sentbytes2 = 0
 
-	def process_queue(self, queue, conns, connsinprogress, server_socket):
+	def process_queue(self, queue, conns, connsinprogress, server_socket, maxsockets=MAXFILELIMIT):
 		""" Processes messages sent by UI thread. server_socket is a server connection
-		socket object, queue holds the messages, conns ans connsinprogess 
+		socket object, queue holds the messages, conns and connsinprogress 
 		are dictionaries holding Connection and PeerConnectionInProgress 
 		messages."""
 		msgList = []
 		needsleep = 0
+		numsockets = 0
+		if server_socket is not None:
+			numsockets += 1
+		numsockets += len(conns) + len(connsinprogress)
 		while not queue.empty():
 			msgList.append(queue.get())
 		for msgObj in msgList:
@@ -788,36 +863,47 @@ class SlskProtoThread(threading.Thread):
 				else:
 					if msgObj.__class__ not in [PeerInit, PierceFireWall, FileSearchResult]:
 						#self._ui_callback([Notify(_("Can't send the message over the closed connection: %s %s") %(msgObj.__class__, vars(msgObj)))])
-						self._ui_callback([_("Can't send the message over the closed connection: %(type)s %(msg_obj)s") %{'type':msgObj.__class__, 'msg_obj':vars(msgObj)}])
+						self._ui_callback([DebugMessage(_("Can't send the message over the closed connection: %(type)s %(msg_obj)s") %{'type':msgObj.__class__, 'msg_obj':vars(msgObj)}, 3)])
 			elif issubclass(msgObj.__class__, InternalMessage):
+				socketwarning = False
 				if msgObj.__class__ is ServerConn:
-					try:
-						server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-						server_socket.setblocking(0)
-						server_socket.connect_ex(msgObj.addr)
-						server_socket.setblocking(1)
-						connsinprogress[server_socket] = PeerConnectionInProgress(server_socket, msgObj)
-					except socket.error, err:
-						self._ui_callback([ConnectError(msgObj, err)])
+					if maxsockets == -1 or numsockets < maxsockets:
+						try:
+							server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+							server_socket.setblocking(0)
+							server_socket.connect_ex(msgObj.addr)
+							server_socket.setblocking(1)
+							connsinprogress[server_socket] = PeerConnectionInProgress(server_socket, msgObj)
+							numsockets += 1
+						except socket.error, err:
+							self._ui_callback([ConnectError(msgObj, err)])
+					else:
+						socketwarning = True
 				elif msgObj.__class__ is ConnClose and msgObj.conn in conns:
 					msgObj.conn.close()
 					#print "Close3", conns[msgObj.conn].addr
 					self._ui_callback([ConnClose(msgObj.conn, conns[msgObj.conn].addr)])
 					del conns[msgObj.conn]
 				elif msgObj.__class__ is OutConn:
-					try:
-						conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-						conn.setblocking(0)
-						conn.connect_ex(msgObj.addr)
-						conn.setblocking(1)
-						connsinprogress[conn] = PeerConnectionInProgress(conn, msgObj)
-					except socket.error, (errnum, strerror):
-						import errno
-						if errno.errorcode.get(errnum,"") == 'EMFILE':
-							queue.put(msgObj)
-							needsleep = 1
-						else:
-							self._ui_callback([ConnectError(msgObj, (errnum, strerror))])
+					if msgObj.addr[1] == 0:
+						self._ui_callback([ConnectError(msgObj, (0, "Port cannot be zero"))])
+					elif maxsockets == -1 or numsockets < maxsockets:
+						try:
+							conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+							conn.setblocking(0)
+							conn.connect_ex(msgObj.addr)
+							conn.setblocking(1)
+							connsinprogress[conn] = PeerConnectionInProgress(conn, msgObj)
+							numsockets += 1
+						except socket.error, (errnum, strerror):
+							import errno
+							if errno.errorcode.get(errnum,"") == 'EMFILE':
+								queue.put(msgObj)
+								needsleep = 1
+							else:
+								self._ui_callback([ConnectError(msgObj, (errnum, strerror))])
+					else:
+						socketwarning = True
 				elif msgObj.__class__ is DownloadFile and msgObj.conn in conns:
 					conns[msgObj.conn].filedown = msgObj
 					conns[msgObj.conn].obuf = conns[msgObj.conn].obuf + struct.pack("<i", msgObj.offset) + struct.pack("<i", 0)
@@ -838,6 +924,9 @@ class SlskProtoThread(threading.Thread):
 						cb = self._calcLimitNone
 					self._resetCounters(conns)
 					self._uploadlimit = (cb, msgObj.limit)
+				if socketwarning and time.time() - self.lastsocketwarning > 60:
+					self.lastsocketwarning = time.time()
+					log.addwarning(_("You have just hit your connection limit of %(limit)s. Nicotine+ will drop connections for your protection. If you get this message often you should search for less generic terms, or increase your per-process file descriptor limit.") % {'limit':maxsockets})
 		if needsleep:
 			time.sleep(1)
 		
