@@ -24,19 +24,28 @@
 transfer manager.
 """
 
+from __future__ import division
+
 import slskmessages
 import threading, thread
 from slskmessages import newId
+from logfacility import log
 
 import os, stat, sys
+import shutil
 import os.path
 import string, re
 import time
 import mp3
 import locale
 import utils
-import md5
-from utils import _
+try:
+	import hashlib
+	MD5 = hashlib.md5
+except:
+	import md5
+	MD5 = md5.new
+from utils import _, executeCommand
 from gtkgui.utils import recode2
 from time import sleep
 import gobject
@@ -65,7 +74,7 @@ class Transfer:
 		self.timequeued = timequeued
 		self.transfertimer = transfertimer
 		self.requestconn = None
-		self.place = place
+		self.place = place # Queue position
 		self.bitrate = bitrate
 		self.length = length
 
@@ -117,11 +126,7 @@ class Transfers:
 		self.users = users
 		self.downloadspanel = None
 		self.uploadspanel = None
-		self.uploadQueueTimer = threading.Timer(60.0, self.checkUploadQueueTimer)
-		self.uploadQueueTimer.start()
 		
-		self.downloadQueueTimer = threading.Timer(180.0, self.checkDowloadQueueTimer)
-		self.downloadQueueTimer.start()
 # queue sizes
 		self.privcount = 0
 		self.oggcount = 0
@@ -156,7 +161,7 @@ class Transfers:
 	def getAddUser(self, msg):
 		""" Server tells us it'll notify us about a change in user's status """
 		if not msg.userexists and self.eventprocessor.config.sections["ui"]["notexists"]:
-			self.eventprocessor.logMessage(_("User %s does not exist") % (msg.user))
+			self.eventprocessor.logMessage(_("User %s does not exist") % (msg.user), 1)
 
 	def GetUserStatus(self, msg):
 		""" We get a status of a user and if he's online, we request a file from him """
@@ -185,10 +190,12 @@ class Transfers:
 
 
 	def getFile(self, user, filename, path="", transfer = None, size=None, bitrate=None, length=None):
-		path=self.CleanPath(path)
+		path=self.CleanPath(path, absolute=True)
 		self.transferFile(0, user, filename, path, transfer, size, bitrate, length)
 
 	def pushFile(self, user, filename, path="", transfer = None, size=None, bitrate=None, length=None ):
+		if size is None:
+			size = self.getFileSize(filename)
 		self.transferFile(1, user, filename, path, transfer, size, bitrate, length)
 
 	def transferFile(self, direction, user, filename, path="", transfer = None, size=None, bitrate=None, length=None):
@@ -209,14 +216,13 @@ class Transfers:
 			status = self.users[user].status
 		else:
 			status = None
-	    
- # Download filter, added by Ceesjan.
+
 		if not direction and self.eventprocessor.config.sections["transfers"]["enablefilters"]:
 			# Only filter downloads, never uploads!
 			try:
 				downloadregexp = re.compile(self.eventprocessor.config.sections["transfers"]["downloadregexp"], re.I)
 				if downloadregexp.search(filename) is not None:
-					self.eventprocessor.logMessage(_("Filtering: %s") % filename)
+					self.eventprocessor.logMessage(_("Filtering: %s") % filename, 5)
 					self.AbortTransfer(transfer)
 					# The string to be displayed on the GUI
 					transfer.status = 'Filtered'
@@ -247,7 +253,7 @@ class Transfers:
 		else:
 			return
 		for i in self.downloads:
-			if i.user == user and i.filename == msg.file and (i.conn is not None or i.status in ["Connection closed by peer", "Establishing connection"]):
+			if i.user == user and i.filename == msg.file and (i.conn is not None or i.status in ["Connection closed by peer", "Establishing connection", "Waiting for download"]):
 				self.AbortTransfer(i)
 				self.getFile(i.user, i.filename, i.path, i)
 				self.eventprocessor.logTransfer(_("Retrying failed download: user %(user)s, file %(file)s") %{'user':i.user, 'file':self.decode(i.filename)}, 1)
@@ -260,30 +266,11 @@ class Transfers:
 			if i.req == req:
 				i.status = "Getting address"
 				self.downloadspanel.update(i)
-				self.startTimeout(i)
 		for i in self.uploads:
 			if i.req == req:
 				i.status = "Getting address"
 				self.uploadspanel.update(i)
-				self.startTimeout(i)
 
-	def startTimeout(self, transfer, delay=30.0):
-		# Request user's details (if not doing so) and start timer
-		
-		if transfer.user not in self.eventprocessor.watchedusers:
-			self.queue.put(slskmessages.AddUser(transfer.user))
-		transfertimeout = TransferTimeout(transfer.req, self.eventprocessor.frame.callback)
-		if transfer.transfertimer is not None:
-			transfer.transfertimer.cancel()
-		if self.eventprocessor.transfers is None:
-			return
-		transfer.transfertimer = threading.Timer(delay, transfertimeout.timeout)
-		try:
-			transfer.transfertimer.start()
-		except thread.error, error:
-			print "Cannot start new timer thread:", error
-
-		
 	def gotAddress(self, req):
 		""" A connection is in progress, we got the address for a user we need
 		to connect to."""
@@ -305,12 +292,10 @@ class Transfers:
 			if i.req == req:
 				i.status = "Waiting for peer to connect"
 				self.downloadspanel.update(i)
-				self.startTimeout(i)
 		for i in self.uploads:
 			if i.req == req:
 				i.status = "Waiting for peer to connect"
 				self.uploadspanel.update(i)
-				self.startTimeout(i)
 
 	def gotCantConnect(self, req):
 		""" We can't connect to the user, either way. """
@@ -344,12 +329,10 @@ class Transfers:
 			if i.req == req:
 				i.status = "Initializing transfer"
 				self.downloadspanel.update(i)
-				self.startTimeout(i)
 		for i in self.uploads:
 			if i.req == req:
 				i.status = "Initializing transfer"
 				self.uploadspanel.update(i)
-				self.startTimeout(i)
 
 	def gotConnect(self, req, conn):
 		""" A connection has been established, now exchange initialisation
@@ -359,13 +342,11 @@ class Transfers:
 				i.status = "Requesting file"
 				i.requestconn = conn
 				self.downloadspanel.update(i)
-				self.startTimeout(i)
 		for i in self.uploads:
 			if i.req == req:
 				i.status = "Requesting file"
 				i.requestconn = conn
 				self.uploadspanel.update(i)
-				self.startTimeout(i)
 	
 	def TransferRequest(self, msg):
 		user = response = None
@@ -381,7 +362,7 @@ class Transfers:
 			conn = None
 			addr = "127.0.0.1"
 		if user is None:
-			self.eventprocessor.logMessage(_("Got transfer request %s but cannot determine requestor") % vars(msg), 1)
+			self.eventprocessor.logMessage(_("Got transfer request %s but cannot determine requestor") % vars(msg), 5)
 			return
 	
 		if msg.direction == 1:
@@ -431,7 +412,7 @@ class Transfers:
 				
 			else:
 				response = slskmessages.TransferResponse(conn, 0, reason = "Cancelled", req = msg.req)
-				self.eventprocessor.logMessage(_("Denied file request: %s") % str(vars(msg)), 1)
+				self.eventprocessor.logMessage(_("Denied file request: %s") % str(vars(msg)), 5)
 		return response
 		
 	def TransferRequestUploads(self, msg, user, conn, addr):
@@ -477,7 +458,7 @@ class Transfers:
 			self.uploads[-1].transfertimer.start()
 			self.uploadspanel.update(self.uploads[-1])
 			
-		self.eventprocessor.logMessage(_("Upload request: %s") % str(vars(msg)), 1)
+		self.eventprocessor.logMessage(_("Upload request: %s") % str(vars(msg)), 5)
 		return response
 
 	def fileIsQueued(self, user, file):
@@ -535,7 +516,7 @@ class Transfers:
 				self.addQueued(user, msg.file)
 			else:
 				self.queue.put(slskmessages.QueueFailed(conn = msg.conn.conn, file = msg.file, reason = "File not shared" ))
-		self.eventprocessor.logMessage(_("Queued upload request: %s") % str(vars(msg)), 1)
+		self.eventprocessor.logMessage(_("Queued upload request: %s") % str(vars(msg)), 5)
 		self.checkUploadQueue()
 
 	def UploadQueueNotification(self, msg):
@@ -663,6 +644,8 @@ class Transfers:
 							i.transfertimer.cancel()
 						self.uploads.remove(i)
 						self.uploadspanel.update()
+					if i in self.downloads:
+						self.eventprocessor.ProcessRequestToPeer(i.user, slskmessages.PlaceInQueueRequest(None, i.filename))
 				self.checkUploadQueue()
 				
 		elif msg.filesize != None:
@@ -685,7 +668,7 @@ class Transfers:
 				self.checkUploadQueue()
 				break
 			else:
-				self.eventprocessor.logMessage(_("Got unknown transfer response: %s") % str(vars(msg)), 1)
+				self.eventprocessor.logMessage(_("Got unknown transfer response: %s") % str(vars(msg)), 5)
 
 	def TransferTimeout(self, msg):
 		for i in (self.downloads+self.uploads)[:]:
@@ -695,6 +678,10 @@ class Transfers:
 				continue
 			i.status = "Cannot connect"
 			i.req = None
+			curtime = time.time()
+			for j in self.uploads:
+				if j.user == i.user:
+					j.timequeued = curtime
 			if i.user not in self.eventprocessor.watchedusers:
 				self.queue.put(slskmessages.AddUser(i.user))
 			self.queue.put(slskmessages.GetUserStatus(i.user))
@@ -724,18 +711,27 @@ class Transfers:
 				try:
 					if not os.access(incompletedir, os.F_OK):
 						os.makedirs(incompletedir)
+					if not os.access(incompletedir, os.R_OK | os.W_OK | os.X_OK):
+						raise OSError, "Download directory %s Permissions error.\nDir Permissions: %s" % (incompletedir, oct( os.stat(incompletedir)[stat.ST_MODE] & 0777))
+						
 				except OSError, strerror:
 					self.eventprocessor.logMessage(_("OS error: %s") % strerror)
 					i.status = "Download directory error"
 					i.conn = None
 					self.queue.put(slskmessages.ConnClose(msg.conn))
+					self.eventprocessor.frame.NewNotification(_("OS error: %s") % strerror, title=_("Nicotine+ :: Directory download error"))
+				
 				else: 
+					
 					# also check for a windows-style incomplete transfer
 					basename = string.split(i.filename,'\\')[-1]
 					basename = self.encode(basename, i.user)
 					winfname = os.path.join(incompletedir, "INCOMPLETE~"+basename)
 					pyfname  = os.path.join(incompletedir, "INCOMPLETE"+basename)
-					pynewfname = os.path.join(incompletedir, "INCOMPLETE"+md5.new(i.filename+i.user).hexdigest()+basename)
+					m = MD5()
+					m.update(i.filename+i.user)
+					
+					pynewfname = os.path.join(incompletedir, "INCOMPLETE"+m.hexdigest()+basename)
 					try:
 						if os.access(winfname, os.F_OK):
 							fname = winfname
@@ -774,11 +770,12 @@ class Transfers:
 						#i.status = "%s" %(str(i.currentbytes))
 						i.status = "Transferring"
 						i.file = f
+						i.place = 0
 						i.offset = size
 						i.starttime = time.time()
-						self.eventprocessor.logMessage(_("Download started: %s") % (u"%s" % f.name))
+						self.eventprocessor.logMessage(_("Download started: %s") % (u"%s" % f.name), 5)
 
-						self.eventprocessor.logTransfer(_("Download started: user %(user)s, file %(file)s") % {'user':i.user, 'file':u"%s" % f.name})
+						self.eventprocessor.logTransfer(_("Download started: user %(user)s, file %(file)s") % {'user':i.user, 'file':u"%s" % f.name}, 5)
 				self.SetIconDownloads()
 				self.downloadspanel.update(i)
 				
@@ -800,7 +797,6 @@ class Transfers:
 					self.queue.put(slskmessages.UploadFile(i.conn, file = f, size = i.size))
 					i.status = "Initializing transfer"
 					i.file = f
-					self.startTimeout(i, delay=60)
 					self.eventprocessor.logTransfer(_("Upload started: user %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)})
 				except IOError, strerror:
 					self.eventprocessor.logMessage(_("Upload I/O error: %s") % strerror)
@@ -819,22 +815,37 @@ class Transfers:
 			self.queue.put(slskmessages.ConnClose(msg.conn))
             
 	def SetIconDownloads(self):
-		if self.eventprocessor.frame.MainNotebook.get_current_page() == 2:
+		frame =  self.eventprocessor.frame
+		if frame.MainNotebook.get_current_page() == frame.MainNotebook.page_num(frame.vboxdownloads):
 			return
-		self.eventprocessor.frame.DownloadsTabLabel.set_image(self.eventprocessor.frame.images["online"])
+		tablabel = frame.GetTabLabel(frame.DownloadsTabLabel)
+		if not tablabel:
+			return
+		tablabel.set_image(frame.images["online"])
 		
 	def SetIconUploads(self):
-		if self.eventprocessor.frame.MainNotebook.get_current_page() == 3:
+		frame =  self.eventprocessor.frame
+		if frame.MainNotebook.get_current_page() == frame.MainNotebook.page_num(frame.vboxuploads):
 			return
-		self.eventprocessor.frame.UploadsTabLabel.set_image(self.eventprocessor.frame.images["online"])
+		tablabel = frame.GetTabLabel(frame.UploadsTabLabel)
+		if not tablabel:
+			return
+		tablabel.set_image(frame.images["online"])
 		
-	def CleanPath(self, path):
+	def CleanPath(self, path, absolute=False):
 		if win32:
+			# Without hacks it is (up to Vista) not possible to have more
+			# than 26 drives mounted, so we can assume a '[a-zA-Z]:\' prefix
+			# for drives - we shouldn't escape that
+			drive = ''
+			if absolute and path[1:3] == ':\\' and path[0:1] and path[0].isalpha():
+				drive = path[:3]
+				path = path[3:]
 			chars = ["?", "\"", ":", ">", "<", "|", "*"]
 			for char in chars:
 				path = path.replace(char, "_")
+			path = ''.join([drive, path])
 		return path
-        
 	def FileDownload(self, msg):
 		""" A file download is in progress"""
 		needupdate = 1
@@ -880,60 +891,57 @@ class Transfers:
 						folder = os.path.join(downloaddir, self.encode(i.path))
 					if not os.access(folder, os.F_OK):
 						os.makedirs(folder)
-					newname = self.getRenamed(os.path.join(folder, basename))
-					try:
-						if win32:
-							os.rename(msg.file.name, u"%s" % newname)
-						else:
-							os.rename(msg.file.name, newname)
-					except OSError:
+					(newname, identicalfile) = self.getRenamedEnhanced(os.path.join(folder, basename), msg.file.name)
+					if newname:
 						try:
-							f1 = open(msg.file.name, "rb")
-							d = f1.read()
-							if win32:
-								f1 = open(u"%s" % newname, "wb")
-							else:
-								f1 = open(newname, "wb")
-							f1.write(d)
-							f1.close()
-							os.remove(msg.file.name)
-						except OSError:
-							self.eventprocessor.logMessage(_("Couldn't move '%(tempfile)s' to '%(file)s'") % {'tempfile':self.decode(msg.file.name), 'file':self.decode(newname)})
+							shutil.move(msg.file.name, newname)
+						except (IOError, OSError), inst:
+								try:
+									shutil.move(msg.file.name, u"%s" % newname)
+								except (IOError, OSError), inst:
+									log.addwarning(_("Couldn't move '%(tempfile)s' to '%(file)s': %(error)s") % {'tempfile':self.decode(msg.file.name), 'file':self.decode(newname), 'error':str(inst)})
 					i.status = "Finished"
-					self.eventprocessor.logMessage(_("Download finished: %(file)s") % {'file':u"%s" % newname})
-					self.eventprocessor.logTransfer(_("Download finished: user %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)})
+					if newname:
+						self.eventprocessor.logMessage(_("Download finished: %(file)s") % {'file':self.decode(newname)}, 5)
+						self.eventprocessor.logTransfer(_("Download finished: user %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)})
+					else:
+						self.eventprocessor.logMessage(_("File %(file)s is identical to %(identical)s, not saving.") % {'file':self.decode(msg.file.name), 'identical':identicalfile})
+						self.eventprocessor.logTransfer(_("Download finished but not saved since it's a duplicate: user %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)})
+						
 					self.queue.put(slskmessages.ConnClose(msg.conn))
 					#if i.speed is not None:
 						#self.queue.put(slskmessages.SendSpeed(i.user, int(i.speed*1024)))
 						#Removed due to misuse. Replaced by SendUploadSpeed
 					i.conn = None
-					if win32:
-						self.addToShared(u"%s" % newname)
-					else:
-						self.addToShared(newname)
-					self.eventprocessor.sendNumSharedFoldersFiles()
+					if newname:
+						if win32:
+							self.addToShared(u"%s" % newname)
+						else:
+							self.addToShared(newname)
+						self.eventprocessor.sendNumSharedFoldersFiles()
 					self.SaveDownloads()
 					self.downloadspanel.update(i)
 					if config["transfers"]["shownotification"]:
 						self.eventprocessor.frame.NewNotification(_("%(file)s downloaded from %(user)s") % {'user':i.user, "file":newname.rsplit(os.sep, 1)[1]}, title=_("Nicotine+ :: file downloaded"))
 
-					if config["transfers"]["afterfinish"]:
-						command = config["transfers"]["afterfinish"].replace("$", utils.escapeCommand(newname))
-						os.system(command)
-						self.eventprocessor.logMessage(_("Executed: %s") % self.decode(command))
-					if i.path and config["transfers"]["shownotification"] or config["transfers"]["afterfolder"]:
+					if newname and config["transfers"]["afterfinish"]:
+						if not executeCommand(config["transfers"]["afterfinish"], newname):
+							self.eventprocessor.logMessage(_("Trouble executing '%s'") % config["transfers"]["afterfinish"])
+						else:
+							self.eventprocessor.logMessage(_("Executed: %s") % config["transfers"]["afterfinish"])
+					if i.path and (config["transfers"]["shownotificationperfolder"] or config["transfers"]["afterfolder"]):
 						# walk through downloads and break if any file in the same folder exists, else execute
 						for ia in self.downloads:
 							if ia.status not in ['Finished', 'Aborted', 'Paused', 'Filtered'] and ia.path and ia.path == i.path:
 								break
 						else:
-							if config["transfers"]["shownotification"]:
+							if config["transfers"]["shownotificationperfolder"]:
 								self.eventprocessor.frame.NewNotification(_("%(folder)s downloaded from %(user)s") % {'user':i.user, "folder":folder}, title=_("Nicotine+ :: directory completed"))
 							if config["transfers"]["afterfolder"]:
-								command = config["transfers"]["afterfolder"].replace("$", utils.escapeCommand(folder))
-								os.system(command)
-								self.eventprocessor.logMessage(_("Executed on folder: %s") % self.decode(command))
-								
+								if not executeCommand(config["transfers"]["afterfolder"], folder):
+									self.eventprocessor.logMessage(_("Trouble executing on folder: %s") % config["transfers"]["afterfolder"])
+								else:
+									self.eventprocessor.logMessage(_("Executed on folder: %s") % config["transfers"]["afterfolder"])
 			except IOError, strerror:
 				self.eventprocessor.logMessage(_("Download I/O error: %s") % self.decode(strerror))
 				i.status = "Local file error"
@@ -1109,14 +1117,6 @@ class Transfers:
 			self.eventprocessor.config.sections["server"]["banlist"].append(user)
 			self.eventprocessor.config.writeConfig()
 	
-	def checkDowloadQueueTimer(self):
-		self.downloadQueueTimer.cancel()
-
-		self.checkDownloadQueue()
-		
-		self.downloadQueueTimer = threading.Timer(180.0, self.checkDowloadQueueTimer)
-		self.downloadQueueTimer.start()
-		
 	# Find failed downloads and attempt to queue them
 	def checkDownloadQueue(self):
 		if self.eventprocessor.config.sections["transfers"]["autoretry_downloads"]:
@@ -1129,14 +1129,6 @@ class Transfers:
 			
 			self.SaveDownloads()
 					
-	def checkUploadQueueTimer(self):
-		self.uploadQueueTimer.cancel()
-
-		self.checkUploadQueue()
-		
-		self.uploadQueueTimer = threading.Timer(60.0, self.checkUploadQueueTimer)
-		self.uploadQueueTimer.start()
-		
 	# Find next file to upload
 	def checkUploadQueue(self):
 		if self.bandwidthLimitReached() or self.transferNegotiating():
@@ -1257,7 +1249,7 @@ class Transfers:
 		sec = int(seconds % 60)
 		minutes = int(seconds / 60 % 60)
 		hours = int(seconds / 3600 % 24)
-		days = int(seconds/86400)
+		days = int(seconds / 86400)
 		
 		time_string = "%02d:%02d:%02d" % (hours, minutes, sec)
 		if days > 0:
@@ -1377,6 +1369,10 @@ class Transfers:
 						self.eventprocessor.logTransfer(_("Retrying failed download: %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)}, 1)
 						
 						self.getFile(i.user, i.filename, i.path, i)
+			curtime = time.time()
+			for j in self.uploads:
+				if j.user == i.user:
+					j.timequeued = curtime
 			i.conn = None
 			self.downloadspanel.update(i)
 			self.uploadspanel.update(i)
@@ -1400,16 +1396,51 @@ class Transfers:
 					break
 				n += 1
 			return newname
-			
+	def getRenamedEnhanced(self, name, originalfile):
+		""" When a transfer is finished, we remove INCOMPLETE~ or INCOMPLETE 
+		prefix from the file's name. 
+		
+		Returns a tuple (newname, identicalfile) where precisely one of the two
+		is None, and the other is a complete path. If newname is None a file
+		with the same checksum value already exists as identicalfile, if
+		identicalfile is None the file can be saved under newname."""
+		if win32 and not os.path.exists(u"%s" % name) and not os.path.exists(name):
+			# Filename doesn't exist, good for renaming
+			return (name, None)
+		elif not win32 and not os.path.exists(name):
+			return (name, None)
+		# A file with the same name already exists. First lets check whether it's a duplicate
+		ourchecksum = self.getChecksum(originalfile)
+		newname = name
+		n = 1
+		while n < 1000:
+			existingchecksum = self.getChecksum(newname)
+			if ourchecksum == existingchecksum:
+				return (None, newname)
+			newname = name+"."+str(n)
+			if not os.path.exists(newname):
+				break
+			n += 1
+		return (newname, None)
+	def getChecksum(self, path):
+		try:
+			h = open(path)
+			m = MD5()
+			m.update(h.read(-1))
+			digest = m.digest()
+			h.close()
+			return digest
+		except IOError:
+			return None
 
 	def PlaceInQueue(self, msg):
 		""" The server tells us our place in queue for a particular transfer."""
-		'''
+		
 		username = None
 		for i in self.peerconns:
-		if i.conn is msg.conn.conn:
-			username = i.username
-			break
+			if i.conn is msg.conn.conn:
+				username = i.username
+				break
 		
 		if username:
 			for i in self.downloads:
@@ -1421,8 +1452,8 @@ class Transfers:
 				#print i.filename.split('\\')[-1] 
 				i.place = msg.place
 				self.downloadspanel.update(i)
-		'''			
-		self.eventprocessor.logMessage(_("File: %(file)s, place in queue: %(place)s") % {'file':self.decode(msg.filename.split('\\')[-1]), 'place':msg.place})
+		
+		#self.eventprocessor.logMessage(_("File: %(file)s, place in queue: %(place)s") % {'file':self.decode(msg.filename.split('\\')[-1]), 'place':msg.place})
 
 	def FileError(self, msg):
 		""" Networking thread encountered a local file error"""
@@ -1454,7 +1485,23 @@ class Transfers:
 		for i in msg.list.keys():
 			for directory in msg.list[i].keys():
 				if os.path.commonprefix([i, directory]) == directory:
-					for file in msg.list[i][directory]:
+					priorityfiles = []
+					normalfiles = []
+					
+					if self.eventprocessor.config.sections["transfers"]["prioritize"]:
+						for file in msg.list[i][directory]:
+							(junk, sep, ext) = file[1].rpartition('.')
+							if sep and ext in ['sfv','md5','nfo']:
+								priorityfiles.append(file)
+							else:
+								normalfiles.append(file)
+					else:
+						normalfiles = msg.list[i][directory][:]
+					if self.eventprocessor.config.sections["transfers"]["reverseorder"]:
+						deco = [(x[1], x) for x in normalfiles]
+						deco.sort(reverse=True)
+						normalfiles = [x for junk, x in deco]
+					for file in priorityfiles + normalfiles:
 						length = bitrate = None
 						attrs = file[4]
 						if attrs != []:
@@ -1465,7 +1512,7 @@ class Transfers:
 								rl = int(attrs[1])
 							except:
 								rl = 0
-							length = "%i:%02i" % (rl / 60, rl % 60)
+							length = "%i:%02i" % (rl // 60, rl % 60)
 						if directory[-1] == '\\':
 							self.getFile(username, directory + file[1], self.FolderDestination(username, directory), size=file[2], bitrate=bitrate, length=length)
 						else:
