@@ -23,33 +23,30 @@
 This module contains utility fuctions.
 """
 
-version = "1.2.9"
+from __future__ import division
+
+version = "1.2.12"
 latesturl = "http://nicotine-plus.sourceforge.net/LATEST"
 
 import string
 from UserDict import UserDict
+from subprocess import Popen, PIPE
 import os, dircache
 import sys
 import gobject
+
+from logfacility import log as logfacility
+
+
 win32 = sys.platform.startswith("win")
 frame = 0
 log = 0
 language = ""
 try:
-	import _mp3 as mp3
-	print "Using C mp3 scanner"
+	import metadata_mutagen as metadata
 except ImportError:
-	import mp3
-
-try:
-	import ogg.vorbis
-	vorbis = ogg.vorbis
-except:
-	try:
-		import _vorbis
-		vorbis = _vorbis
-	except:
-		vorbis = None
+	logfacility.addwarning("Failed to import the Mutagen library, falling back to old library. To improve meta data please install Mutagen.")
+	import mp3 as metadata
 
 import gettext
 tr_cache = {}
@@ -69,11 +66,33 @@ def ChangeTranslation(lang):
 		langTranslation = gettext.translation('nicotine', languages=[language])
 		langTranslation.install()
 	except IOError, e:
-		message = _("Translation not found for '%s': %s") % (language, e)
+		message = _("Translation not found for '%(language)s': %(error)s") % {'language':language, 'error':e}
 		langTranslation = gettext
 	except IndexError, e:
-		message = _("Translation was corrupted for '%s': %s") % (language, e)
+		message = _("Translation was corrupted for '%(language)s': %(error)s") % {'language':language, 'error':e}
 		langTranslation = gettext
+	else:
+		# We need to force LC_ALL env variable to get buttons translated as well
+		# On Windows we can't use os.environ, see pynicotine/libi18n.py
+		if win32:
+			import libi18n
+			libi18n._putenv('LC_ALL',language)
+			del libi18n
+		# The Unix part is quite funny too...
+		else:
+			import locale
+			# Let's try to force the locale with the right lang
+			# locale.normalize will return bla_BLA.USUAL_ENCODINGS, so we first want to try to use unicode
+			try:
+				locale.setlocale(locale.LC_ALL, locale.normalize(language).split('.')[0]+'.UTF-8')
+			except locale.Error, e:
+				# If it fails we'll trust normalize() and use its encoding
+				try:
+					locale.setlocale(locale.LC_ALL, locale.normalize(language))
+				# Sorry, please generate the right locales
+				except locale.Error, e:
+					message = _("Force lang failed: '%(language)s' (%(second)s and %(third)s tested)" ) % {'language':e, 'second':locale.normalize(language).split('.')[0]+'.UTF-8', 'third':locale.normalize(language)}
+			del locale
 	return message
 ChangeTranslation(language)
 # Translation Function
@@ -107,38 +126,41 @@ def getServerList(url):
 		return []
 
 # Rescan directories in shared databases
-def rescandirs(shared, sharedmtimes, sharedfiles, sharedfilesstreams, yieldfunction, progress=None, name="", rebuild=False):
-	# Check for modified or new files
-	# returns dict in format:  { Directory : mtime, ... }
+def rescandirs(shared_directories, oldmtimes, oldfiles, sharedfilesstreams, yieldfunction, progress=None, name="", rebuild=False):
+	"""
+	Check for modified or new files via OS's last mtime on a directory,
+	or, if rebuild is True, all directories
+	"""
+	#returns dict in format:  { Directory : mtime, ... }
 	
 	gobject.idle_add(progress.set_text, _("Checking for changes"))
 	gobject.idle_add(progress.show)
 	gobject.idle_add(progress.set_fraction, 0)
-	
+
 	if win32:
-		newmtimes = getDirsMtimesUnicode(shared, yieldfunction)
+		newmtimes = getDirsMtimesUnicode(shared_directories, yieldfunction)
 	else:
-		newmtimes = getDirsMtimes(shared, yieldfunction)
+		newmtimes = getDirsMtimes(shared_directories, yieldfunction)
 	gobject.idle_add(progress.set_text, _("Scanning %s") % name)
 	# Get list of files
 	# returns dict in format { Directory : { File : metadata, ... }, ... }
 	if win32:
-		newsharedfiles = getFilesListUnicode(newmtimes, sharedmtimes, sharedfiles,yieldfunction, progress, rebuild)
+		newsharedfiles = getFilesListUnicode(newmtimes, oldmtimes, oldfiles, yieldfunction, progress, rebuild)
 	else:
-		newsharedfiles = getFilesList(newmtimes, sharedmtimes, sharedfiles,yieldfunction, progress, rebuild)
+		newsharedfiles = getFilesList(newmtimes, oldmtimes, oldfiles, yieldfunction, progress, rebuild)
 	# Pack shares data
 	# returns dict in format { Directory : hex string of files+metadata, ... }
 	gobject.idle_add(progress.set_text, _("Building DataBase"))
-	newsharedfilesstreams = getFilesStreams(newmtimes, sharedmtimes, sharedfilesstreams, newsharedfiles, yieldfunction)
+	newsharedfilesstreams = getFilesStreams(newmtimes, oldmtimes, sharedfilesstreams, newsharedfiles, yieldfunction)
 	
 	# Update Search Index
 	# newwordindex is a dict in format {word: [num, num, ..], ... } with num matching
 	# keys in newfileindex
 	# newfileindex is a dict in format { num: (path, size, (bitrate, vbr), length), ... }
 	gobject.idle_add(progress.set_text, _("Building Index"))
-	newwordindex, newfileindex = getFilesIndex(newmtimes, sharedmtimes, shared, newsharedfiles, yieldfunction)
+	newwordindex, newfileindex = getFilesIndex(newmtimes, oldmtimes, shared_directories, newsharedfiles, yieldfunction)
 	gobject.idle_add(progress.set_fraction, 1.0)
-	
+
 	return newsharedfiles, newsharedfilesstreams, newwordindex, newfileindex, newmtimes
     
 
@@ -161,7 +183,7 @@ def getDirsMtimesUnicode(dirs, yieldcall = None):
 				contents = os.listdir(u_directory)
 				mtime = os.path.getmtime(str_directory)
 		except OSError, errtuple:
-			message = _("Scanning Directory Error: %s Path: %s") % (errtuple, u_directory)
+			message = _("Scanning Directory Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':u_directory}
 			print str(message)
 			if log:
 				log(message)
@@ -179,7 +201,7 @@ def getDirsMtimesUnicode(dirs, yieldcall = None):
 			try:
 				isdir = os.path.isdir(u_path)
 			except OSError, errtuple:
-				message = _("Scanning Error: %s Path: %s") % (errtuple, u_path)
+				message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':u_path}
 				print str(message)
 				if log:
 					log(message)
@@ -190,7 +212,7 @@ def getDirsMtimesUnicode(dirs, yieldcall = None):
 				try:
 					mtime = os.path.getmtime(s_path)
 				except OSError, errtuple:
-					message = _("Scanning Error: %s Path: %s") % (errtuple, u_path)
+					message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':u_path}
 					print str(message)
 					if log:
 						log(message)
@@ -218,12 +240,13 @@ def getDirsMtimes(dirs, yieldcall = None):
 			contents = dircache.listdir(directory)
 			mtime = os.path.getmtime(directory)
 		except OSError, errtuple:
-			message = _("Scanning Directory Error: %s Path: %s") % (errtuple, directory)
+			message = _("Scanning Directory Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':directory}
 			print str(message)
 			if log:
 				log(message)
 			displayTraceback(sys.exc_info()[2])
 			continue
+
 		list[directory] = mtime
 
 		for filename in contents:
@@ -233,7 +256,7 @@ def getDirsMtimes(dirs, yieldcall = None):
 			try:
 				isdir = os.path.isdir(path)
 			except OSError, errtuple:
-				message = _("Scanning Error: %s Path: %s") % (errtuple, path)
+				message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':path}
 				print str(message)
 				if log:
 					log(message)
@@ -241,7 +264,15 @@ def getDirsMtimes(dirs, yieldcall = None):
 			try:
 				mtime = os.path.getmtime(path)
 			except OSError, errtuple:
-				message = _("Scanning Error: %s Path: %s") % (errtuple, path)
+				islink = False
+				try:
+					islink = os.path.islink(path)
+				except OSError, errtuple2:
+					print errtuple2
+				if islink:
+					message = _("Scanning Error: Broken link to directory: \"%(link)s\" from Path: \"%(path)s\". Repair or remove this link.") % {'link':os.readlink(path), 'path':path}
+				else:
+					message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':path}
 				print str(message)
 				if log:
 					log(message)
@@ -275,8 +306,13 @@ def getFilesList(mtimes, oldmtimes, oldlist, yieldcall = None, progress=None, re
 			continue
 		if not rebuild and directory in oldmtimes:
 			if mtimes[directory] == oldmtimes[directory]:
-				list[directory] = oldlist[directory]
-				continue
+				if os.path.exists(directory):
+					
+					list[directory] = oldlist[directory]
+					continue
+				else:
+					print "Dropping removed directory %s" % directory
+					continue
 
 		list[directory] = []
 
@@ -297,7 +333,7 @@ def getFilesList(mtimes, oldmtimes, oldlist, yieldcall = None, progress=None, re
 
 				isfile = os.path.isfile(path)
 			except OSError, errtuple:
-				message = _("Scanning Error: %s Path: %s") % (errtuple, path)
+				message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':path}
 				print str(message)
 				if log:
 					log(message)
@@ -362,7 +398,7 @@ def getFilesListUnicode(mtimes, oldmtimes, oldlist, yieldcall = None, progress=N
 				# try to force Unicode for reading from disk
 				isfile = os.path.isfile(ppath)
 			except OSError, errtuple:
-				message = _("Scanning Error: %s Path: %s") % (errtuple, ppath)
+				message = _("Scanning Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':path}
 				print str(message)
 				if log:
 					log(message)
@@ -392,34 +428,18 @@ def getFileInfoUnicode(name, pathname):
 		except:
 			size = os.path.getsize(pathname)
 			
-		if name[-4:].lower() == ".mp3":
-			try:
-				mp3info = mp3.detect_mp3(pathname_f)
-			except:
-				mp3info = mp3.detect_mp3(pathname)
-			if mp3info:
-				bitrateinfo = (mp3info["bitrate"], mp3info["vbr"])
-				fileinfo = (name, size, bitrateinfo, mp3info["time"])
-			else:
-				fileinfo = (name, size, None, None)
-		
-		elif vorbis and (name[-4:].lower() == ".ogg"):
-
-			try:
-				try:
-					vf = vorbis.VorbisFile(pathname_f)
-				except:
-					vf = vorbis.VorbisFile(pathname)
-				time = int(vf.time_total(0))
-				bitrate = vf.bitrate(0)/1000
-				fileinfo = (name, size, (bitrate, 0), time)
-			except:
-				fileinfo = (name, size, None, None)
+		try:
+			info = metadata.detect(pathname_f)
+		except:
+			info = metadata.detect(pathname)
+		if info:
+			bitrateinfo = (int(info["bitrate"]), int(info["vbr"]))
+			fileinfo = (name, size, bitrateinfo, int(info["time"]))
 		else:
 			fileinfo = (name, size, None, None)
 		return fileinfo
 	except Exception, errtuple:
-		message = _("Scanning File Error: %s Path: %s") % (errtuple, pathname)
+		message = _("Scanning File Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':pathname}
 		if log:
 			log(message)
 		displayTraceback(sys.exc_info()[2])
@@ -428,42 +448,36 @@ def getFileInfoUnicode(name, pathname):
 def getFileInfo(name, pathname):
 	try:
 		size = os.path.getsize(pathname)
-			
-		if name[-4:].lower() == ".mp3":
-			mp3info = mp3.detect_mp3(pathname)
-			if mp3info:
-				bitrateinfo = (mp3info["bitrate"], mp3info["vbr"])
-				fileinfo = (name, size, bitrateinfo, mp3info["time"])
-			else:
-				fileinfo = (name, size, None, None)
-		elif vorbis and (name[-4:].lower() == ".ogg"):
-			try:
-				vf = vorbis.VorbisFile(pathname)
-				time = int(vf.time_total(0))
-				bitrate = vf.bitrate(0)/1000
-				fileinfo = (name, size, (bitrate, 0), time)
-			except:
-				fileinfo = (name, size, None, None)
+		info = metadata.detect(pathname)
+		if info:
+			bitrateinfo = (int(info["bitrate"]), int(info["vbr"]))
+			fileinfo = (name, size, bitrateinfo, int(info["time"]))
 		else:
 			fileinfo = (name, size, None, None)
 		return fileinfo
 	except Exception, errtuple:
-		message = _("Scanning File Error: %s Path: %s") % (errtuple, pathname)
+		message = _("Scanning File Error: %(error)s Path: %(path)s") % {'error':errtuple, 'path':pathname}
 		if log:
 			log(message)
 		displayTraceback(sys.exc_info()[2])
 		
-def getFilesStreams(mtimes, oldmtimes, oldstreams, sharedfiles, yieldcall = None):
+def getFilesStreams(mtimes, oldmtimes, oldstreams, newsharedfiles, yieldcall = None):
 	streams = {}
-	for i in mtimes.keys():
-		if hiddenCheck(i):
-			continue	
-		if i in oldmtimes:
-			if mtimes[i] == oldmtimes[i]:
-				# No change
-				streams[i] = oldstreams[i]
-				continue
-		streams[i] = getDirStream(sharedfiles[i])
+	for directory in mtimes.keys():
+		if hiddenCheck(directory):
+			continue
+
+		if directory in oldmtimes:
+			if mtimes[directory] == oldmtimes[directory]:
+				if os.path.exists(directory):
+					# No change
+					streams[directory] = oldstreams[directory]
+					continue
+				else:
+					print "Dropping missing directory %s" % directory
+					continue
+				
+		streams[directory] = getDirStream(newsharedfiles[directory])
 		if yieldcall is not None:
 			yieldcall()
 	return streams
@@ -506,20 +520,20 @@ def getByteStream(fileinfo):
 	return stream
 
 # Update Search index with new files
-def getFilesIndex(mtimes, oldmtimes, shareddirs, sharedfiles, yieldcall = None):
+def getFilesIndex(mtimes, oldmtimes, shareddirs, newsharedfiles, yieldcall = None):
 	wordindex = {}
 	fileindex = {}
 	index = 0
 	
-	for i in mtimes.keys():
+	for directory in mtimes.keys():
 		
-		if hiddenCheck(i):
+		if hiddenCheck(directory):
 			continue
-		for j in sharedfiles[i]:
-			indexes = getIndexWords(i, j[0], shareddirs)
+		for j in newsharedfiles[directory]:
+			indexes = getIndexWords(directory, j[0], shareddirs)
 			for k in indexes:
 				wordindex.setdefault(k, []).append(index)
-			fileindex[str(index)] = (os.path.join(i, j[0]), )+j[1:]
+			fileindex[str(index)] = (os.path.join(directory, j[0]), )+j[1:]
 			index += 1
 		if yieldcall is not None:
 			yieldcall()
@@ -609,3 +623,82 @@ class SortedDict(UserDict):
 			self.__sorted__ = True
 		for key in self.__keys__:
 			yield key, self[key]
+
+def executeCommand(command, replacement=None, background=True, returnoutput=False, placeholder='$'):
+	"""Executes a string with commands, with partial support for bash-style quoting and pipes
+	
+	The different parts of the command should be separated by spaces, a double
+	quotation mark can be used to embed spaces in an argument. Pipes can be created
+	using the bar symbol (|).
+
+	If background is false the function will wait for all the launches processes to end
+	before returning.
+
+	If the 'replacement' argument is given, every occurance of 'placeholder'
+	will be replaced by 'replacement'.
+	
+	If the command ends with the ampersand symbol background will be set to True. This should
+	only be done by the request of the user, if you want background to be true set the function
+	argument.
+
+	The only expected error to be thrown is the RuntimeError in case something goes wrong
+	while executing the command.
+
+	Example commands:
+	* "C:\Program Files\WinAmp\WinAmp.exe" --xforce "--title=My Window Title" 
+	* mplayer $
+	* echo $ | flite -t """
+	# Example command: "C:\Program Files\WinAmp\WinAmp.exe" --xforce "--title=My Title" $ | flite -t
+	if returnoutput:
+		background = False
+	command = command.strip()
+	if command.endswith("&"):
+		command = command[:-1]
+		if returnoutput:
+			print "Yikes, I was asked to return output but I'm also asked to launch the process in the background. returnoutput gets precedent."
+		else:
+			background = True
+	unparsed = command
+	arguments = []
+	while unparsed.count('"') > 1:
+		(pre, argument, post) = unparsed.split('"', 2)
+		if pre:
+			arguments += pre.rstrip(' ').split(' ')
+		arguments.append(argument)
+		unparsed = post.lstrip(' ')
+	if unparsed:
+		arguments += unparsed.split(' ')
+	# arguments is now: ['C:\Program Files\WinAmp\WinAmp.exe', '--xforce', '--title=My Title', '$', '|', 'flite', '-t']
+	subcommands = []
+	current = []
+	for argument in arguments:
+		if argument in ('|',):
+			subcommands.append(current)
+			current = []
+		else:
+			current.append(argument)
+	subcommands.append(current)
+	# subcommands is now: [['C:\Program Files\WinAmp\WinAmp.exe', '--xforce', '--title=My Title', '$'], ['flite', '-t']]
+	if replacement:
+		for i in xrange(0, len(subcommands)):
+			subcommands[i] = [x.replace(placeholder, replacement) for x in subcommands[i]]
+	# Chaining commands...
+	finalstdout = None
+	if returnoutput:
+		finalstdout = PIPE
+	procs = []
+	try:
+		if len(subcommands) == 1: # no need to fool around with pipes
+			procs.append(Popen(subcommands[0], stdout=finalstdout))
+		else:
+			procs.append(Popen(subcommands[0], stdout=PIPE))
+			for subcommand in subcommands[1:-1]:
+				procs.append(Popen(subcommand, stdin=procs[-1].stdout, stdout=PIPE))
+			procs.append(Popen(subcommands[-1], stdin=procs[-1].stdout, stdout=finalstdout))
+		if not background and not returnoutput:
+			procs[-1].wait()
+	except:
+		raise RuntimeError("Problem while executing command %s (%s of %s)" % (subcommands[len(procs)], len(procs)+1, len(subcommands)))
+	if not returnoutput:
+		return True
+	return procs[-1].communicate()[0]
